@@ -109,10 +109,10 @@ gst_kate_parse_class_init (GstKateParseClass * klass)
 
   gstelement_class->change_state = gst_kate_parse_change_state;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_kate_parse_src_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_kate_parse_sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_kate_parse_src_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_kate_parse_sink_factory);
 
   gst_element_class_set_static_metadata (gstelement_class, "Kate stream parser",
       "Codec/Parser/Subtitle",
