@@ -1,7 +1,7 @@
 /*
  * GStreamer
  * Copyright (C) 2011 Robert Jobbagy <jobbagy.robert@gmail.com>
- * Copyright (C) 2011 - 2018 Nicola Murino <nicola.murino@gmail.com>
+ * Copyright (C) 2011 Nicola Murino <nicola.murino@gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -45,7 +45,13 @@
 #ifndef MOTIONCELLS_H_
 #define MOTIONCELLS_H_
 
-#include <opencv2/core.hpp>
+#include <opencv2/core/core_c.h>
+#ifdef HAVE_HIGHGUI_H
+#include <highgui.h>            // includes highGUI definitions
+#endif
+#ifdef HAVE_OPENCV2_HIGHGUI_HIGHGUI_C_H
+#include <opencv2/highgui/highgui_c.h>            // includes highGUI definitions
+#endif
 #include <fstream>
 #include <vector>
 #include <glib.h>
@@ -104,18 +110,18 @@ struct Cell
 
 struct MotionCellsIdx
 {
-  cv::Rect motioncell;
+  CvRect motioncell;
   //Points for the edges of the rectangle.
-  cv::Point cell_pt1;
-  cv::Point cell_pt2;
+  CvPoint cell_pt1;
+  CvPoint cell_pt2;
   int lineidx;
   int colidx;
 };
 
 struct OverlayRegions
 {
-  cv::Point upperleft;
-  cv::Point lowerright;
+  CvPoint upperleft;
+  CvPoint lowerright;
 };
 
 class MotionCells
@@ -125,7 +131,7 @@ public:
   MotionCells ();
   virtual ~ MotionCells ();
 
-  int performDetectionMotionCells (cv::Mat p_frame, double p_sensitivity,
+  int performDetectionMotionCells (IplImage * p_frame, double p_sensitivity,
       double p_framerate, int p_gridx, int p_gridy, gint64 timestamp_millisec,
       bool p_isVisble, bool p_useAlpha, int motionmaskcoord_count,
       motionmaskcoordrect * motionmaskcoords, int motionmaskcells_count,
@@ -133,9 +139,9 @@ public:
       int motioncells_count, motioncellidx * motioncellsidx, gint64 starttime,
       char *datafile, bool p_changed_datafile, int p_thickness);
 
-  void setPrevFrame (cv::Mat p_prevframe)
+  void setPrevFrame (IplImage * p_prevframe)
   {
-    m_pprevFrame = p_prevframe.clone();
+    m_pprevFrame = cvCloneImage (p_prevframe);
   }
   char *getMotionCellsIdx ()
   {
@@ -193,26 +199,24 @@ private:
       p_motionmaskcellsidx, int p_motionmaskcells_count = 0);
   int saveMotionCells (gint64 timestamp_millisec);
   int initDataFile (char *p_datafile, gint64 starttime);
-  void blendImages (cv::Mat p_actFrame, cv::Mat p_cellsFrame,
+  void blendImages (IplImage * p_actFrame, IplImage * p_cellsFrame,
       float p_alpha, float p_beta);
 
-  void setData (cv::Mat img, int lin, int col, uchar valor)
+  void setData (IplImage * img, int lin, int col, uchar valor)
   {
-    ((uchar *) (img.data + img.step[0] * lin))[col] = valor;
+    ((uchar *) (img->imageData + img->widthStep * lin))[col] = valor;
   }
 
-  uchar getData (cv::Mat img, int lin, int col)
+  uchar getData (IplImage * img, int lin, int col)
   {
-    return ((uchar *) (img.data + img.step[0] * lin))[col];
+    return ((uchar *) (img->imageData + img->widthStep * lin))[col];
   }
 
-  bool getIsNonZero (cv::Mat img)
+  bool getIsNonZero (IplImage * img)
   {
-    int lin, col;
-
-    for (lin = 0; lin < img.size().height; lin++)
-      for (col = 0; col < img.size().width; col++) {
-        if ((((uchar *) (img.data + img.step[0] * lin))[col]) > 0)
+    for (int lin = 0; lin < img->height; lin++)
+      for (int col = 0; col < img->width; col++) {
+        if ((((uchar *) (img->imageData + img->widthStep * lin))[col]) > 0)
           return true;
       }
     return false;
@@ -220,17 +224,15 @@ private:
 
   void setMotionCells (int p_frameWidth, int p_frameHeight)
   {
-    int i, j;
-
     m_cellwidth = (double) p_frameWidth / (double) m_gridx;
     m_cellheight = (double) p_frameHeight / (double) m_gridy;
     m_pCells = new Cell *[m_gridy];
-    for (i = 0; i < m_gridy; i++)
+    for (int i = 0; i < m_gridy; i++)
       m_pCells[i] = new Cell[m_gridx];
 
     //init cells
-    for (i = 0; i < m_gridy; i++)
-      for (j = 0; j < m_gridx; j++) {
+    for (int i = 0; i < m_gridy; i++)
+      for (int j = 0; j < m_gridx; j++) {
         m_pCells[i][j].MotionArea = 0;
         m_pCells[i][j].CellArea = 0;
         m_pCells[i][j].MotionPercent = 0;
@@ -238,8 +240,8 @@ private:
       }
   }
 
-  cv::Mat m_pprevFrame, m_pdifferenceImage, m_pbwImage, m_pcurFrame,
-    transparencyimg;
+  IplImage *m_pcurFrame, *m_pprevFrame, *m_pdifferenceImage,
+      *m_pbwImage,*transparencyimg;
   bool m_isVisible, m_changed_datafile, m_useAlpha, m_saveInDatafile;
   Cell **m_pCells;
   vector < MotionCellsIdx > m_MotionCells;
