@@ -7,7 +7,7 @@
  *  and adapted. Its license reads:
  *  "Oct. 3, 2008
  *   Right to use this code in any way you want without warrenty, support or
- *   any guarantee of it working. "
+ *   any guarentee of it working. "
  *
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -72,17 +72,18 @@
  * mixture model for real-time tracking with shadow detection", Proc. 2nd
  * European Workshop on Advanced Video-Based Surveillance Systems, 2001
  * [5] http://opencv.itseez.com/modules/video/doc/motion_analysis_and_object_tracking.html#backgroundsubtractormog2
- * [6] Z.Zivkovic, "Improved adaptive Gaussian mixture model for background
+ * [6] Z.Zivkovic, "Improved adaptive Gausian mixture model for background
  * subtraction", International Conference Pattern Recognition, UK, August, 2004.
  * [7] Z.Zivkovic, F. van der Heijden, "Efficient Adaptive Density Estimation
  * per Image Pixel for the Task of Background Subtraction", Pattern Recognition
  * Letters, vol. 27, no. 7, pages 773-780, 2006.
  *
- * ## Example launch line
- *
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0  v4l2src device=/dev/video0 ! videoconvert ! segmentation test-mode=true method=2 ! videoconvert ! ximagesink
+ * gst-launch-1.0  v4l2src device=/dev/video0 ! videoconvert ! video/x-raw,width=320,height=240 ! videoconvert ! segmentation test-mode=true method=2 ! videoconvert ! ximagesink
  * ]|
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -90,13 +91,15 @@
 #endif
 
 #include "gstsegmentation.h"
-#include <opencv2/imgproc.hpp>
+#include <opencv2/imgproc/imgproc_c.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_segmentation_debug);
 #define GST_CAT_DEFAULT gst_segmentation_debug
 
 using namespace cv;
-
+#if (CV_MAJOR_VERSION >= 3)
+  using namespace cv::bgsegm;
+#endif
 /* Filter signals and args */
 enum
 {
@@ -139,14 +142,7 @@ gst_segmentation_method_get_type (void)
   return etype;
 }
 
-G_DEFINE_TYPE_WITH_CODE (GstSegmentation, gst_segmentation,
-    GST_TYPE_OPENCV_VIDEO_FILTER,
-    GST_DEBUG_CATEGORY_INIT (gst_segmentation_debug, "segmentation", 0,
-        "Performs Foreground/Background segmentation in video sequences");
-    );
-GST_ELEMENT_REGISTER_DEFINE (segmentation, "segmentation", GST_RANK_NONE,
-    GST_TYPE_SEGMENTATION);
-
+G_DEFINE_TYPE (GstSegmentation, gst_segmentation, GST_TYPE_VIDEO_FILTER);
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -158,20 +154,19 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("RGBA")));
 
 
-static void
-gst_segmentation_set_property (GObject * object, guint prop_id,
+static void gst_segmentation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
-static void
-gst_segmentation_get_property (GObject * object, guint prop_id,
+static void gst_segmentation_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static GstFlowReturn gst_segmentation_transform_ip (GstOpencvVideoFilter *
-    filter, GstBuffer * buffer, Mat img);
+static GstFlowReturn gst_segmentation_transform_ip (GstVideoFilter * btrans,
+    GstVideoFrame * frame);
 
-static void gst_segmentation_finalize (GObject * object);
-static gboolean gst_segmentation_set_caps (GstOpencvVideoFilter * filter,
-    gint in_width, gint in_height, int in_cv_type, gint out_width,
-    gint out_height, int out_cv_type);
+static gboolean gst_segmentation_stop (GstBaseTransform * basesrc);
+static gboolean gst_segmentation_set_info (GstVideoFilter * filter,
+    GstCaps * incaps, GstVideoInfo * in_info,
+    GstCaps * outcaps, GstVideoInfo * out_info);
+static void gst_segmentation_release_all_pointers (GstSegmentation * filter);
 
 /* Codebook algorithm + connected components functions*/
 static int update_codebook (unsigned char *p, codeBook * c,
@@ -179,12 +174,14 @@ static int update_codebook (unsigned char *p, codeBook * c,
 static int clear_stale_entries (codeBook * c);
 static unsigned char background_diff (unsigned char *p, codeBook * c,
     int numChannels, int *minMod, int *maxMod);
-static void find_connected_components (Mat mask, int poly1_hull0,
-    float perimScale);
+static void find_connected_components (IplImage * mask, int poly1_hull0,
+    float perimScale, CvMemStorage * mem_storage, CvSeq * contours);
 
 /* MOG (Mixture-of-Gaussians functions */
+static int initialise_mog (GstSegmentation * filter);
 static int run_mog_iteration (GstSegmentation * filter);
 static int run_mog2_iteration (GstSegmentation * filter);
+static int finalise_mog (GstSegmentation * filter);
 
 /* initialize the segmentation's class */
 static void
@@ -192,18 +189,18 @@ gst_segmentation_class_init (GstSegmentationClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  GstOpencvVideoFilterClass *cvfilter_class =
-      (GstOpencvVideoFilterClass *) klass;
+  GstBaseTransformClass *basesrc_class = GST_BASE_TRANSFORM_CLASS (klass);
+  GstVideoFilterClass *video_class = (GstVideoFilterClass *) klass;
 
   gobject_class = (GObjectClass *) klass;
 
-  gobject_class->finalize = gst_segmentation_finalize;
   gobject_class->set_property = gst_segmentation_set_property;
   gobject_class->get_property = gst_segmentation_get_property;
 
+  basesrc_class->stop = gst_segmentation_stop;
 
-  cvfilter_class->cv_trans_ip_func = gst_segmentation_transform_ip;
-  cvfilter_class->cv_set_caps = gst_segmentation_set_caps;
+  video_class->transform_frame_ip = gst_segmentation_transform_ip;
+  video_class->set_info = gst_segmentation_set_info;
 
   g_object_class_install_property (gobject_class, PROP_METHOD,
       g_param_spec_enum ("method",
@@ -232,12 +229,11 @@ gst_segmentation_class_init (GstSegmentationClass * klass)
   gst_element_class_add_static_pad_template (element_class, &src_factory);
   gst_element_class_add_static_pad_template (element_class, &sink_factory);
 
-  gst_type_mark_as_plugin_api (GST_TYPE_SEGMENTATION_METHOD, (GstPluginAPIFlags) 0);
 }
 
 /* initialize the new element
  * instantiate pads and add them to element
- * set pad callback functions
+ * set pad calback functions
  * initialize instance structure
  */
 static void
@@ -247,9 +243,10 @@ gst_segmentation_init (GstSegmentation * filter)
   filter->test_mode = DEFAULT_TEST_MODE;
   filter->framecount = 0;
   filter->learning_rate = DEFAULT_LEARNING_RATE;
-  gst_opencv_video_filter_set_in_place (GST_OPENCV_VIDEO_FILTER (filter), TRUE);
+  gst_base_transform_set_in_place (GST_BASE_TRANSFORM (filter), TRUE);
 }
 
+
 static void
 gst_segmentation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -294,26 +291,35 @@ gst_segmentation_get_property (GObject * object, guint prop_id,
   }
 }
 
+/* GstElement vmethod implementations */
+/* this function handles the link with other elements */
 static gboolean
-gst_segmentation_set_caps (GstOpencvVideoFilter * filter, gint in_width,
-    gint in_height, int in_cv_type,
-    gint out_width, gint out_height, int out_cv_type)
+gst_segmentation_set_info (GstVideoFilter * filter,
+    GstCaps * incaps, GstVideoInfo * in_info,
+    GstCaps * outcaps, GstVideoInfo * out_info)
 {
   GstSegmentation *segmentation = GST_SEGMENTATION (filter);
-  Size size;
+  CvSize size;
 
-  size = Size (in_width, in_height);
-  segmentation->width = in_width;
-  segmentation->height = in_height;
+  size = cvSize (in_info->width, in_info->height);
+  segmentation->width = in_info->width;
+  segmentation->height = in_info->height;
+  /*  If cvRGB is already allocated, it means there's a cap modification, */
+  /*  so release first all the images. */
+  if (NULL != segmentation->cvRGBA)
+    gst_segmentation_release_all_pointers (segmentation);
 
-  segmentation->cvRGB.create (size, CV_8UC3);
-  segmentation->cvYUV.create (size, CV_8UC3);
+  segmentation->cvRGBA = cvCreateImageHeader (size, IPL_DEPTH_8U, 4);
 
-  segmentation->cvFG = Mat::zeros (size, CV_8UC1);
+  segmentation->cvRGB = cvCreateImage (size, IPL_DEPTH_8U, 3);
+  segmentation->cvYUV = cvCreateImage (size, IPL_DEPTH_8U, 3);
 
-  segmentation->ch1.create (size, CV_8UC1);
-  segmentation->ch2.create (size, CV_8UC1);
-  segmentation->ch3.create (size, CV_8UC1);
+  segmentation->cvFG = cvCreateImage (size, IPL_DEPTH_8U, 1);
+  cvZero (segmentation->cvFG);
+
+  segmentation->ch1 = cvCreateImage (size, IPL_DEPTH_8U, 1);
+  segmentation->ch2 = cvCreateImage (size, IPL_DEPTH_8U, 1);
+  segmentation->ch3 = cvCreateImage (size, IPL_DEPTH_8U, 1);
 
   /* Codebook method */
   segmentation->TcodeBook = (codeBook *)
@@ -326,43 +332,54 @@ gst_segmentation_set_caps (GstOpencvVideoFilter * filter, gint in_width,
   segmentation->learning_interval = (int) (1.0 / segmentation->learning_rate);
 
   /* Mixture-of-Gaussians (mog) methods */
-  segmentation->mog = bgsegm::createBackgroundSubtractorMOG ();
-  segmentation->mog2 = createBackgroundSubtractorMOG2 ();
+  initialise_mog (segmentation);
 
   return TRUE;
 }
 
 /* Clean up */
-static void
-gst_segmentation_finalize (GObject * object)
+static gboolean
+gst_segmentation_stop (GstBaseTransform * basesrc)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (object);
+  GstSegmentation *filter = GST_SEGMENTATION (basesrc);
+
+  if (filter->cvRGBA != NULL)
+    gst_segmentation_release_all_pointers (filter);
+
+  return TRUE;
+}
+
+static void
+gst_segmentation_release_all_pointers (GstSegmentation * filter)
+{
+  cvReleaseImage (&filter->cvRGBA);
+  cvReleaseImage (&filter->cvRGB);
+  cvReleaseImage (&filter->cvYUV);
+  cvReleaseImage (&filter->cvFG);
+  cvReleaseImage (&filter->ch1);
+  cvReleaseImage (&filter->ch2);
+  cvReleaseImage (&filter->ch3);
+
+  cvReleaseMemStorage (&filter->mem_storage);
 
-  filter->cvRGB.release ();
-  filter->cvYUV.release ();
-  filter->cvFG.release ();
-  filter->ch1.release ();
-  filter->ch2.release ();
-  filter->ch3.release ();
-  filter->mog.release ();
-  filter->mog2.release ();
   g_free (filter->TcodeBook);
-
-  G_OBJECT_CLASS (gst_segmentation_parent_class)->finalize (object);
+  finalise_mog (filter);
 }
 
 static GstFlowReturn
-gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
-    GstBuffer * buffer, Mat img)
+gst_segmentation_transform_ip (GstVideoFilter * btrans, GstVideoFrame * frame)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (cvfilter);
+  GstSegmentation *filter = GST_SEGMENTATION (btrans);
   int j;
 
+  /*  get image data from the input, which is RGBA */
+  filter->cvRGBA->imageData = (char *) GST_VIDEO_FRAME_COMP_DATA (frame, 0);
+  filter->cvRGBA->widthStep = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0);
   filter->framecount++;
 
   /*  Image preprocessing: color space conversion etc */
-  cvtColor (img, filter->cvRGB, COLOR_RGBA2RGB);
-  cvtColor (filter->cvRGB, filter->cvYUV, COLOR_RGB2YCrCb);
+  cvCvtColor (filter->cvRGBA, filter->cvRGB, CV_RGBA2RGB);
+  cvCvtColor (filter->cvRGB, filter->cvYUV, CV_RGB2YCrCb);
 
   /* Create and update a fg/bg model using a codebook approach following the
    * opencv O'Reilly book [1] implementation of the algo described in [2].
@@ -374,20 +391,19 @@ gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
   if (METHOD_BOOK == filter->method) {
     unsigned cbBounds[3] = { 10, 5, 5 };
     int minMod[3] = { 20, 20, 20 }, maxMod[3] = {
-      20, 20, 20
-    };
+    20, 20, 20};
 
     if (filter->framecount < 30) {
       /* Learning background phase: update_codebook on every frame */
       for (j = 0; j < filter->width * filter->height; j++) {
-        update_codebook (filter->cvYUV.data + j * 3,
+        update_codebook ((unsigned char *) filter->cvYUV->imageData + j * 3,
             (codeBook *) & (filter->TcodeBook[j]), cbBounds, 3);
       }
     } else {
       /*  this updating is responsible for FG becoming BG again */
       if (filter->framecount % filter->learning_interval == 0) {
         for (j = 0; j < filter->width * filter->height; j++) {
-          update_codebook (filter->cvYUV.data + j * 3,
+          update_codebook ((uchar *) filter->cvYUV->imageData + j * 3,
               (codeBook *) & (filter->TcodeBook[j]), cbBounds, 3);
         }
       }
@@ -398,17 +414,18 @@ gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
 
       for (j = 0; j < filter->width * filter->height; j++) {
         if (background_diff
-            (filter->cvYUV.data + j * 3,
+            ((uchar *) filter->cvYUV->imageData + j * 3,
                 (codeBook *) & (filter->TcodeBook[j]), 3, minMod, maxMod)) {
-          filter->cvFG.data[j] = (char) 255;
+          filter->cvFG->imageData[j] = 255;
         } else {
-          filter->cvFG.data[j] = 0;
+          filter->cvFG->imageData[j] = 0;
         }
       }
     }
 
     /* 3rd param is the smallest area to show: (w+h)/param , in pixels */
-    find_connected_components (filter->cvFG, 1, 10000);
+    find_connected_components (filter->cvFG, 1, 10000,
+        filter->mem_storage, filter->contours);
 
   }
   /* Create the foreground and background masks using BackgroundSubtractorMOG [1],
@@ -428,7 +445,7 @@ gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
    * OpenCV MOG2 implements the algorithm described in [2] and [3].
    *
    * [1] http://opencv.itseez.com/modules/video/doc/motion_analysis_and_object_tracking.html#backgroundsubtractormog2
-   * [2] Z.Zivkovic, "Improved adaptive Gaussian mixture model for background
+   * [2] Z.Zivkovic, "Improved adaptive Gausian mixture model for background
    * subtraction", International Conference Pattern Recognition, UK, Aug 2004.
    * [3] Z.Zivkovic, F. van der Heijden, "Efficient Adaptive Density Estimation
    * per Image Pixel for the Task of Background Subtraction", Pattern
@@ -438,24 +455,35 @@ gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
   }
 
   /*  if we want to test_mode, just overwrite the output */
-  std::vector < cv::Mat > channels (3);
-
   if (filter->test_mode) {
-    cvtColor (filter->cvFG, filter->cvRGB, COLOR_GRAY2RGB);
+    cvCvtColor (filter->cvFG, filter->cvRGB, CV_GRAY2RGB);
 
-    split (filter->cvRGB, channels);
+    cvSplit (filter->cvRGB, filter->ch1, filter->ch2, filter->ch3, NULL);
   } else
-    split (img, channels);
-
-  channels.push_back (filter->cvFG);
+    cvSplit (filter->cvRGBA, filter->ch1, filter->ch2, filter->ch3, NULL);
 
   /*  copy anyhow the fg/bg to the alpha channel in the output image */
-  merge (channels, img);
+  cvMerge (filter->ch1, filter->ch2, filter->ch3, filter->cvFG, filter->cvRGBA);
 
 
   return GST_FLOW_OK;
 }
 
+/* entry point to initialize the plug-in
+ * initialize the plug-in itself
+ * register the element factories and other features
+ */
+gboolean
+gst_segmentation_plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (gst_segmentation_debug, "segmentation",
+      0, "Performs Foreground/Background segmentation in video sequences");
+
+  return gst_element_register (plugin, "segmentation", GST_RANK_NONE,
+      GST_TYPE_SEGMENTATION);
+}
+
+
 
 #ifdef CODE_FROM_OREILLY_BOOK   /* See license at the beginning of the page */
 /*
@@ -622,7 +650,7 @@ clear_stale_entries (codeBook * c)
   maxMod Add this (possibly negative) number onto
 
   max level when determining if new pixel is foreground
-  minMod Subtract this (possibly negative) number from
+  minMod Subract this (possibly negative) number from
   min level when determining if new pixel is foreground
 
   NOTES:
@@ -691,53 +719,89 @@ background_diff (unsigned char *p, codeBook * c, int numChannels,
 /* How many iterations of erosion and/or dilation there should be */
 #define CVCLOSE_ITR 1
 static void
-find_connected_components (Mat mask, int poly1_hull0, float perimScale)
+find_connected_components (IplImage * mask, int poly1_hull0, float perimScale,
+    CvMemStorage * mem_storage, CvSeq * contours)
 {
+  CvContourScanner scanner;
+  CvSeq *c;
+  int numCont = 0;
   /* Just some convenience variables */
-  const Scalar CVX_WHITE = CV_RGB (0xff, 0xff, 0xff);
-  //const Scalar CVX_BLACK = CV_RGB (0x00, 0x00, 0x00);
-  int idx = 0;
+  const CvScalar CVX_WHITE = CV_RGB (0xff, 0xff, 0xff);
+  const CvScalar CVX_BLACK = CV_RGB (0x00, 0x00, 0x00);
 
   /* CLEAN UP RAW MASK */
-  morphologyEx (mask, mask, MORPH_OPEN, Mat (), Point (-1, -1), CVCLOSE_ITR);
-  morphologyEx (mask, mask, MORPH_CLOSE, Mat (), Point (-1, -1), CVCLOSE_ITR);
+  cvMorphologyEx (mask, mask, 0, 0, CV_MOP_OPEN, CVCLOSE_ITR);
+  cvMorphologyEx (mask, mask, 0, 0, CV_MOP_CLOSE, CVCLOSE_ITR);
   /* FIND CONTOURS AROUND ONLY BIGGER REGIONS */
+  if (mem_storage == NULL) {
+    mem_storage = cvCreateMemStorage (0);
+  } else {
+    cvClearMemStorage (mem_storage);
+  }
 
-  std::vector < std::vector < Point > >contours;
-  std::vector < std::vector < Point > >to_draw;
-  std::vector < Vec4i > hierarchy;
-  findContours (mask, contours, hierarchy, RETR_TREE, CHAIN_APPROX_SIMPLE,
-      Point (0, 0));
-  if (contours.size () == 0)
-    return;
+  scanner = cvStartFindContours (mask, mem_storage, sizeof (CvContour),
+      CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, cvPoint (0, 0));
 
-  for (; idx >= 0; idx = hierarchy[idx][0]) {
-    const std::vector < Point > &c = contours[idx];
-    double len = fabs (contourArea (Mat (c)));
-    double q = (mask.size ().height + mask.size ().width) / perimScale;
-    if (len >= q) {
-      std::vector < Point > c_new;
+  while ((c = cvFindNextContour (scanner)) != NULL) {
+    double len = cvContourArea (c, CV_WHOLE_SEQ, 0);
+    /* calculate perimeter len threshold: */
+    double q = (mask->height + mask->width) / perimScale;
+    /* Get rid of blob if its perimeter is too small: */
+    if (len < q) {
+      cvSubstituteContour (scanner, NULL);
+    } else {
+      /* Smooth its edges if its large enough */
+      CvSeq *c_new;
       if (poly1_hull0) {
-        approxPolyDP (c, c_new, CVCONTOUR_APPROX_LEVEL, (hierarchy[idx][2] < 0
-                && hierarchy[idx][3] < 0));
+        /* Polygonal approximation */
+        c_new =
+            cvApproxPoly (c, sizeof (CvContour), mem_storage, CV_POLY_APPROX_DP,
+            CVCONTOUR_APPROX_LEVEL, 0);
       } else {
-        convexHull (c, c_new, true, true);
+        /* Convex Hull of the segmentation */
+        c_new = cvConvexHull2 (c, mem_storage, CV_CLOCKWISE, 1);
       }
-      to_draw.push_back (c_new);
+      cvSubstituteContour (scanner, c_new);
+      numCont++;
     }
   }
+  contours = cvEndFindContours (&scanner);
 
-  mask.setTo (Scalar::all (0));
-  if (to_draw.size () > 0) {
-    drawContours (mask, to_draw, -1, CVX_WHITE, FILLED);
-  }
-
+  /* PAINT THE FOUND REGIONS BACK INTO THE IMAGE */
+  cvZero (mask);
+  /* DRAW PROCESSED CONTOURS INTO THE MASK */
+  for (c = contours; c != NULL; c = c->h_next)
+    cvDrawContours (mask, c, CVX_WHITE, CVX_BLACK, -1, CV_FILLED, 8, cvPoint (0,
+            0));
 }
 #endif /*ifdef CODE_FROM_OREILLY_BOOK */
 
+
+int
+initialise_mog (GstSegmentation * filter)
+{
+  filter->img_input_as_cvMat = (void *) new Mat (cvarrToMat (filter->cvYUV, false));
+  filter->img_fg_as_cvMat = (void *) new Mat (cvarrToMat(filter->cvFG, false));
+
+#if (CV_MAJOR_VERSION >= 3)
+  filter->mog = bgsegm::createBackgroundSubtractorMOG ();
+  filter->mog2 = createBackgroundSubtractorMOG2 ();
+#else
+  filter->mog = (void *) new BackgroundSubtractorMOG ();
+  filter->mog2 = (void *) new BackgroundSubtractorMOG2 ();
+#endif
+
+  return (0);
+}
+
 int
 run_mog_iteration (GstSegmentation * filter)
 {
+  ((cv::Mat *) filter->img_input_as_cvMat)->data =
+      (uchar *) filter->cvYUV->imageData;
+  ((cv::Mat *) filter->img_fg_as_cvMat)->data =
+      (uchar *) filter->cvFG->imageData;
+
   /*
      BackgroundSubtractorMOG [1], Gaussian Mixture-based Background/Foreground
      Segmentation Algorithm. OpenCV MOG implements the algorithm described in [2].
@@ -748,7 +812,15 @@ run_mog_iteration (GstSegmentation * filter)
      European Workshop on Advanced Video-Based Surveillance Systems, 2001
    */
 
-  filter->mog->apply (filter->cvYUV, filter->cvFG, filter->learning_rate);
+#if (CV_MAJOR_VERSION >= 3)
+  filter->mog->apply (*((Mat *) filter->
+          img_input_as_cvMat), *((Mat *) filter->img_fg_as_cvMat),
+      filter->learning_rate);
+#else
+  (*((BackgroundSubtractorMOG *) filter->mog)) (*((Mat *) filter->
+          img_input_as_cvMat), *((Mat *) filter->img_fg_as_cvMat),
+      filter->learning_rate);
+#endif
 
   return (0);
 }
@@ -756,20 +828,48 @@ run_mog_iteration (GstSegmentation * filter)
 int
 run_mog2_iteration (GstSegmentation * filter)
 {
+  ((Mat *) filter->img_input_as_cvMat)->data =
+       (uchar *) filter->cvYUV->imageData;
+  ((Mat *) filter->img_fg_as_cvMat)->data =
+       (uchar *) filter->cvFG->imageData;
+
   /*
      BackgroundSubtractorMOG2 [1], Gaussian Mixture-based Background/Foreground
      segmentation algorithm. OpenCV MOG2 implements the algorithm described in
      [2] and [3].
 
      [1] http://opencv.itseez.com/modules/video/doc/motion_analysis_and_object_tracking.html#backgroundsubtractormog2
-     [2] Z.Zivkovic, "Improved adaptive Gaussian mixture model for background
+     [2] Z.Zivkovic, "Improved adaptive Gausian mixture model for background
      subtraction", International Conference Pattern Recognition, UK, August, 2004.
      [3] Z.Zivkovic, F. van der Heijden, "Efficient Adaptive Density Estimation per
      Image Pixel for the Task of Background Subtraction", Pattern Recognition
      Letters, vol. 27, no. 7, pages 773-780, 2006.
    */
 
-  filter->mog2->apply (filter->cvYUV, filter->cvFG, filter->learning_rate);
+#if (CV_MAJOR_VERSION >= 3)
+  filter->mog2->apply (*((Mat *) filter->
+          img_input_as_cvMat), *((Mat *) filter->img_fg_as_cvMat),
+      filter->learning_rate);
+#else
+  (*((BackgroundSubtractorMOG *) filter->mog2)) (*((Mat *) filter->
+          img_input_as_cvMat), *((Mat *) filter->img_fg_as_cvMat),
+      filter->learning_rate);
+#endif
 
   return (0);
 }
+
+int
+finalise_mog (GstSegmentation * filter)
+{
+  delete (Mat *) filter->img_input_as_cvMat;
+  delete (Mat *) filter->img_fg_as_cvMat;
+#if (CV_MAJOR_VERSION >= 3)
+  filter->mog.release ();
+  filter->mog2.release ();
+#else
+  delete (BackgroundSubtractorMOG *) filter->mog;
+  delete (BackgroundSubtractorMOG2 *) filter->mog2;
+#endif
+  return (0);
+}
