@@ -159,15 +159,10 @@ enum
   PROP_RTCP_AUTH,
   PROP_RANDOM_KEY,
   PROP_REPLAY_WINDOW_SIZE,
-  PROP_ALLOW_REPEAT_TX
+  PROP_ALLOW_REPEAT_TX,
+  PROP_STATS
 };
 
-typedef struct ValidateBufferItData
-{
-  GstSrtpEnc *filter;
-  gboolean is_rtcp;
-} ValidateBufferItData;
-
 typedef struct ProcessBufferItData
 {
   GstSrtpEnc *filter;
@@ -252,27 +247,6 @@ static GstPad *gst_srtp_enc_request_new_pad (GstElement * element,
 static void gst_srtp_enc_release_pad (GstElement * element, GstPad * pad);
 
 
-static guint32
-gst_srtp_enc_get_rollover_counter (GstSrtpEnc * filter, guint32 ssrc)
-{
-  guint32 roc = 0;
-  srtp_stream_t stream;
-
-  GST_OBJECT_LOCK (filter);
-
-  GST_DEBUG_OBJECT (filter, "retrieving SRTP Rollover Counter, ssrc: %u", ssrc);
-
-  if (filter->session) {
-    stream = srtp_get_stream (filter->session, htonl (ssrc));
-    if (stream)
-      roc = stream->rtp_rdbx.index >> 16;
-  }
-
-  GST_OBJECT_UNLOCK (filter);
-
-  return roc;
-}
-
 /* initialize the srtpenc's class
  */
 static void
@@ -284,14 +258,14 @@ gst_srtp_enc_class_init (GstSrtpEncClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&rtp_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&rtp_sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&rtcp_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&rtcp_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &rtp_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &rtp_sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &rtcp_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &rtcp_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class, "SRTP encoder",
       "Filter/Network/SRTP",
@@ -348,6 +322,9 @@ gst_srtp_enc_class_init (GstSrtpEncClass * klass)
           "(Note that such repeated transmissions must have the same RTP payload, "
           "or a severe security weakness is introduced!)",
           DEFAULT_ALLOW_REPEAT_TX, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics", "Various statistics",
+          GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstSrtpEnc::soft-limit:
@@ -360,22 +337,6 @@ gst_srtp_enc_class_init (GstSrtpEncClass * klass)
   gst_srtp_enc_signals[SIGNAL_SOFT_LIMIT] =
       g_signal_new ("soft-limit", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
-
-  /**
-   * GstSrtpEnc::get-rollover-counter:
-   * @gstsrtpenc: the element on which the signal is emitted
-   * @ssrc: The unique SSRC of the stream
-   *
-   * Request the SRTP rollover counter for the stream with @ssrc.
-   */
-  gst_srtp_enc_signals[SIGNAL_GET_ROLLOVER_COUNTER] =
-      g_signal_new ("get-rollover-counter", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstSrtpEncClass,
-          get_rollover_counter), NULL, NULL, g_cclosure_marshal_generic,
-      G_TYPE_UINT, 1, G_TYPE_UINT);
-
-  klass->get_rollover_counter =
-      GST_DEBUG_FUNCPTR (gst_srtp_enc_get_rollover_counter);
 }
 
 
@@ -425,9 +386,11 @@ gst_srtp_enc_create_session (GstSrtpEnc * filter)
     gsize keysize;
 
     if (filter->key == NULL) {
+      GST_OBJECT_UNLOCK (filter);
       GST_ELEMENT_ERROR (filter, LIBRARY, SETTINGS,
           ("Cipher is not NULL, key must be set"),
           ("Cipher is not NULL, key must be set"));
+      GST_OBJECT_LOCK (filter);
       return err_status_fail;
     }
 
@@ -435,10 +398,12 @@ gst_srtp_enc_create_session (GstSrtpEnc * filter)
     keysize = gst_buffer_get_size (filter->key);
 
     if (expected != keysize) {
+      GST_OBJECT_UNLOCK (filter);
       GST_ELEMENT_ERROR (filter, LIBRARY, SETTINGS,
           ("Master key size is wrong"),
           ("Expected master key of %d bytes, but received %" G_GSIZE_FORMAT
               " bytes", expected, keysize));
+      GST_OBJECT_LOCK (filter);
       return err_status_fail;
     }
   }
@@ -639,6 +604,41 @@ gst_srtp_enc_dispose (GObject * object)
   G_OBJECT_CLASS (gst_srtp_enc_parent_class)->dispose (object);
 }
 
+static GstStructure *
+gst_srtp_enc_create_stats (GstSrtpEnc * filter)
+{
+  GstStructure *s;
+  GValue va = G_VALUE_INIT;
+  GValue v = G_VALUE_INIT;
+
+  s = gst_structure_new_empty ("application/x-srtp-encoder-stats");
+
+  g_value_init (&va, GST_TYPE_ARRAY);
+  g_value_init (&v, GST_TYPE_STRUCTURE);
+
+  if (filter->session) {
+    srtp_stream_t stream = filter->session->stream_list;
+    while (stream) {
+      GstStructure *ss;
+      guint32 ssrc = GUINT32_FROM_BE (stream->ssrc);
+      guint32 roc = stream->rtp_rdbx.index >> 16;
+
+      ss = gst_structure_new ("application/x-srtp-stream",
+          "ssrc", G_TYPE_UINT, ssrc, "roc", G_TYPE_UINT, roc, NULL);
+
+      g_value_take_boxed (&v, ss);
+      gst_value_array_append_value (&va, &v);
+
+      stream = stream->next;
+    }
+  }
+
+  gst_structure_take_value (s, "streams", &va);
+  g_value_unset (&v);
+
+  return s;
+}
+
 static void
 gst_srtp_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -730,6 +730,9 @@ gst_srtp_enc_get_property (GObject * object, guint prop_id,
     case PROP_ALLOW_REPEAT_TX:
       g_value_set_boolean (value, filter->allow_repeat_tx);
       break;
+    case PROP_STATS:
+      g_value_take_boxed (value, gst_srtp_enc_create_stats (filter));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -970,34 +973,6 @@ gst_srtp_enc_replace_random_key (GstSrtpEnc * filter)
   filter->key_changed = TRUE;
 }
 
-static gboolean
-gst_srtp_enc_check_buffer (GstSrtpEnc * filter, GstBuffer * buf,
-    gboolean is_rtcp)
-{
-  if (!is_rtcp) {
-    GstRTPBuffer rtpbuf = GST_RTP_BUFFER_INIT;
-
-    if (!gst_rtp_buffer_map (buf, GST_MAP_READ, &rtpbuf)) {
-      GST_ELEMENT_ERROR (filter, STREAM, WRONG_TYPE, (NULL),
-          ("Could not map RTP buffer"));
-      return FALSE;
-    }
-
-    gst_rtp_buffer_unmap (&rtpbuf);
-  } else {
-    GstRTCPBuffer rtcpbuf = GST_RTCP_BUFFER_INIT;
-
-    if (!gst_rtcp_buffer_map (buf, GST_MAP_READ, &rtcpbuf)) {
-      GST_ELEMENT_ERROR (filter, STREAM, WRONG_TYPE, (NULL),
-          ("Could not map RTCP buffer"));
-      return FALSE;
-    }
-    gst_rtcp_buffer_unmap (&rtcpbuf);
-  }
-
-  return TRUE;
-}
-
 static GstFlowReturn
 gst_srtp_enc_check_set_caps (GstSrtpEnc * filter, GstPad * pad,
     gboolean is_rtcp)
@@ -1109,10 +1084,6 @@ gst_srtp_enc_chain (GstPad * pad, GstObject * parent, GstBuffer * buf,
   GstPad *otherpad;
   GstBuffer *bufout = NULL;
 
-  if (!gst_srtp_enc_check_buffer (filter, buf, is_rtcp)) {
-    goto fail;
-  }
-
   if ((ret = gst_srtp_enc_check_set_caps (filter, pad, is_rtcp)) != GST_FLOW_OK) {
     goto out;
   }
@@ -1162,19 +1133,6 @@ fail:
   goto out;
 }
 
-static gboolean
-validate_buffer_it (GstBuffer ** buffer, guint index, gpointer user_data)
-{
-  ValidateBufferItData *data = user_data;
-
-  if (!gst_srtp_enc_check_buffer (data->filter, *buffer, data->is_rtcp)) {
-    GST_WARNING_OBJECT (data->filter, "Invalid buffer, dropping");
-    gst_buffer_replace (buffer, NULL);
-  }
-
-  return TRUE;
-}
-
 static gboolean
 process_buffer_it (GstBuffer ** buffer, guint index, gpointer user_data)
 {
@@ -1200,17 +1158,11 @@ gst_srtp_enc_chain_list (GstPad * pad, GstObject * parent,
   GstFlowReturn ret = GST_FLOW_OK;
   GstPad *otherpad;
   GstBufferList *out_list = NULL;
-  ValidateBufferItData validate_data;
   ProcessBufferItData process_data;
 
-  validate_data.filter = filter;
-  validate_data.is_rtcp = is_rtcp;
-
   GST_LOG_OBJECT (pad, "Buffer chain with list of %d",
       gst_buffer_list_length (buf_list));
 
-  gst_buffer_list_foreach (buf_list, validate_buffer_it, &validate_data);
-
   if (!gst_buffer_list_length (buf_list))
     goto out;
 
