@@ -19,7 +19,7 @@
  */
 
 /**
- * SECTION:vkwindow
+ * SECTION:gstglwindow
  * @short_description: window/surface abstraction
  * @title: GstVulkanWindow
  * @see_also: #GstGLContext, #GstGLDisplay
@@ -50,16 +50,18 @@
 #define GST_CAT_DEFAULT gst_vulkan_window_debug
 GST_DEBUG_CATEGORY (GST_CAT_DEFAULT);
 
+#define gst_vulkan_window_parent_class parent_class
+G_DEFINE_ABSTRACT_TYPE (GstVulkanWindow, gst_vulkan_window, GST_TYPE_OBJECT);
+
+#define GST_VULKAN_WINDOW_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_VULKAN_WINDOW, GstVulkanWindowPrivate))
+
 struct _GstVulkanWindowPrivate
 {
   guint surface_width;
   guint surface_height;
 };
 
-#define gst_vulkan_window_parent_class parent_class
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GstVulkanWindow, gst_vulkan_window,
-    GST_TYPE_OBJECT);
-
 static void gst_vulkan_window_finalize (GObject * object);
 
 typedef struct _GstVulkanDummyWindow
@@ -130,12 +132,16 @@ _init_debug (void)
 static void
 gst_vulkan_window_init (GstVulkanWindow * window)
 {
-  window->priv = gst_vulkan_window_get_instance_private (window);
+  window->priv =
+      G_TYPE_INSTANCE_GET_PRIVATE (window, GST_TYPE_VULKAN_WINDOW,
+      GstVulkanWindowPrivate);
 }
 
 static void
 gst_vulkan_window_class_init (GstVulkanWindowClass * klass)
 {
+  g_type_class_add_private (klass, sizeof (GstVulkanWindowPrivate));
+
   klass->open = GST_DEBUG_FUNCPTR (gst_vulkan_window_default_open);
   klass->close = GST_DEBUG_FUNCPTR (gst_vulkan_window_default_close);
 
@@ -220,9 +226,9 @@ gst_vulkan_window_get_surface (GstVulkanWindow * window, GError ** error)
 {
   GstVulkanWindowClass *klass;
 
-  g_return_val_if_fail (GST_IS_VULKAN_WINDOW (window), (VkSurfaceKHR) 0);
+  g_return_val_if_fail (GST_IS_VULKAN_WINDOW (window), NULL);
   klass = GST_VULKAN_WINDOW_GET_CLASS (window);
-  g_return_val_if_fail (klass->get_surface != NULL, (VkSurfaceKHR) 0);
+  g_return_val_if_fail (klass->get_surface != NULL, NULL);
 
   return klass->get_surface (window, error);
 }
@@ -305,10 +311,5 @@ gst_vulkan_dummy_window_init (GstVulkanDummyWindow * dummy)
 GstVulkanDummyWindow *
 gst_vulkan_dummy_window_new (void)
 {
-  GstVulkanDummyWindow *window;
-
-  window = g_object_new (gst_vulkan_dummy_window_get_type (), NULL);
-  gst_object_ref_sink (window);
-
-  return window;
+  return g_object_new (gst_vulkan_dummy_window_get_type (), NULL);
 }
