@@ -20,13 +20,9 @@
 
 /**
  * SECTION:gstjpeg2000sampling
- * @title: GstJpeg2000Sampling
  * @short_description: Manage JPEG 2000 sampling and colorspace fields
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "gstjpeg2000sampling.h"
 
@@ -49,8 +45,7 @@ GstJPEG2000Sampling
 gst_jpeg2000_sampling_from_string (const gchar * sampling_string)
 {
   GstJPEG2000Sampling i;
-  if (sampling_string == NULL)
-    return GST_JPEG2000_SAMPLING_NONE;
+  g_return_val_if_fail (sampling_string != NULL, GST_JPEG2000_SAMPLING_NONE);
   for (i = 0; i < G_N_ELEMENTS (gst_jpeg2000_sampling_strings); ++i) {
     if (!g_strcmp0 (sampling_string, gst_jpeg2000_sampling_strings[i]))
       return (i + 1);
