@@ -40,15 +40,17 @@
 #define _GNU_SOURCE
 #endif
 
-#ifndef GL_NUM_EXTENSIONS
-#define GL_NUM_EXTENSIONS 0x0000821d
-#endif
-
 #include <gmodule.h>
+#include <string.h>
+#include <stdio.h>
 
 #include "gl.h"
 #include "gstglcontext.h"
 
+#ifndef GL_NUM_EXTENSIONS
+#define GL_NUM_EXTENSIONS 0x0000821d
+#endif
+
 #if GST_GL_HAVE_PLATFORM_GLX
 #include "x11/gstglcontext_glx.h"
 #endif
@@ -65,7 +67,8 @@
 #include "eagl/gstglcontext_eagl.h"
 #endif
 
-GST_DEBUG_CATEGORY_STATIC (gst_performance);
+extern void _gst_gl_debug_enable (GstGLContext * context);
+
 static GPrivate current_context_key;
 
 static GModule *module_self;
@@ -83,7 +86,7 @@ load_opengl_module (gpointer user_data)
 #else
   /* On Linux the .so is only in -dev packages, try with a real soname
    * Proper compilers will optimize away the strcmp */
-  if (strcmp (G_MODULE_SUFFIX, "so") == 0)
+  if (g_strcmp0 (G_MODULE_SUFFIX, "so") == 0)
     module_opengl = g_module_open ("libGL.so.1", G_MODULE_BIND_LAZY);
 
   /* This automatically handles the suffix and even .la files */
@@ -108,7 +111,7 @@ load_gles2_module (gpointer user_data)
 #else
   /* On Linux the .so is only in -dev packages, try with a real soname
    * Proper compilers will optimize away the strcmp */
-  if (strcmp (G_MODULE_SUFFIX, "so") == 0)
+  if (g_strcmp0 (G_MODULE_SUFFIX, "so") == 0)
     module_gles2 = g_module_open ("libGLESv2.so.2", G_MODULE_BIND_LAZY);
 
   /* This automatically handles the suffix and even .la files */
@@ -129,12 +132,55 @@ load_self_module (gpointer user_data)
   return NULL;
 }
 
-#if GST_GL_HAVE_GLES3
-#error "Add module loading support for GLES3"
-#endif
+/* Context sharedness is tracked by a refcounted pointer stored in each context
+ * object to track complex creation/deletion scenarios.  As a result,
+ * sharedness can only be successfully validated between two GstGLContext's
+ * where one is not a wrapped context.
+ *
+ * As there is no API at the winsys level to tell whether two OpenGL contexts
+ * can share GL resources, this is the next best thing.
+ *
+ * XXX: we may need a way to associate two wrapped GstGLContext's as being
+ * shared however I have not come across a use case that requries this yet.
+ */
+struct ContextShareGroup
+{
+  volatile int refcount;
+};
+
+static struct ContextShareGroup *
+_context_share_group_new (void)
+{
+  struct ContextShareGroup *ret = g_new0 (struct ContextShareGroup, 1);
+
+  ret->refcount = 1;
+
+  return ret;
+}
+
+static struct ContextShareGroup *
+_context_share_group_ref (struct ContextShareGroup *share)
+{
+  g_atomic_int_inc (&share->refcount);
+  return share;
+}
+
+static void
+_context_share_group_unref (struct ContextShareGroup *share)
+{
+  if (g_atomic_int_dec_and_test (&share->refcount))
+    g_free (share);
+}
+
+static gboolean
+_context_share_group_is_shared (struct ContextShareGroup *share)
+{
+  return g_atomic_int_get (&share->refcount) > 1;
+}
 
 #define GST_CAT_DEFAULT gst_gl_context_debug
 GST_DEBUG_CATEGORY (GST_CAT_DEFAULT);
+GST_DEBUG_CATEGORY_STATIC (gst_gl_debug);
 
 #define gst_gl_context_parent_class parent_class
 G_DEFINE_ABSTRACT_TYPE (GstGLContext, gst_gl_context, GST_TYPE_OBJECT);
@@ -145,26 +191,22 @@ G_DEFINE_ABSTRACT_TYPE (GstGLContext, gst_gl_context, GST_TYPE_OBJECT);
 static void _init_debug (void);
 
 static gpointer gst_gl_context_create_thread (GstGLContext * context);
-static gpointer _default_get_proc_address (GstGLContext * context,
-    const gchar * name);
 static void gst_gl_context_finalize (GObject * object);
 
 struct _GstGLContextPrivate
 {
-  GstGLDisplay *display;
-
   GThread *gl_thread;
   GThread *active_thread;
 
   /* conditions */
   GMutex render_lock;
   GCond create_cond;
-  GCond destroy_cond;
 
   gboolean created;
   gboolean alive;
 
   GWeakRef other_context_ref;
+  struct ContextShareGroup *sharegroup;
   GError **error;
 
   gint gl_major;
@@ -194,8 +236,8 @@ G_DEFINE_TYPE (GstGLWrappedContext, gst_gl_wrapped_context,
 
 #define GST_GL_WRAPPED_CONTEXT(o)           (G_TYPE_CHECK_INSTANCE_CAST((o), GST_GL_TYPE_WRAPPED_CONTEXT, GstGLWrappedContext))
 #define GST_GL_WRAPPED_CONTEXT_CLASS(k)     (G_TYPE_CHECK_CLASS((k), GST_GL_TYPE_CONTEXT, GstGLContextClass))
-#define GST_GL_IS_WRAPPED_CONTEXT(o)        (G_TYPE_CHECK_INSTANCE_TYPE((o), GST_GL_TYPE_WRAPPED_CONTEXT))
-#define GST_GL_IS_WRAPPED_CONTEXT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE((k), GST_GL_TYPE_WRAPPED_CONTEXT))
+#define GST_IS_GL_WRAPPED_CONTEXT(o)        (G_TYPE_CHECK_INSTANCE_TYPE((o), GST_GL_TYPE_WRAPPED_CONTEXT))
+#define GST_IS_GL_WRAPPED_CONTEXT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE((k), GST_GL_TYPE_WRAPPED_CONTEXT))
 #define GST_GL_WRAPPED_CONTEXT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS((o), GST_GL_TYPE_WRAPPED_CONTEXT, GstGLWrappedContextClass))
 
 GQuark
@@ -212,7 +254,7 @@ _ensure_window (GstGLContext * context)
   if (context->window)
     return;
 
-  window = gst_gl_window_new (context->priv->display);
+  window = gst_gl_window_new (context->display);
 
   gst_gl_context_set_window (context, window);
 
@@ -230,7 +272,6 @@ gst_gl_context_init (GstGLContext * context)
   g_mutex_init (&context->priv->render_lock);
 
   g_cond_init (&context->priv->create_cond);
-  g_cond_init (&context->priv->destroy_cond);
   context->priv->created = FALSE;
 
   g_weak_ref_init (&context->priv->other_context_ref, NULL);
@@ -241,7 +282,8 @@ gst_gl_context_class_init (GstGLContextClass * klass)
 {
   g_type_class_add_private (klass, sizeof (GstGLContextPrivate));
 
-  klass->get_proc_address = GST_DEBUG_FUNCPTR (_default_get_proc_address);
+  klass->get_proc_address =
+      GST_DEBUG_FUNCPTR (gst_gl_context_default_get_proc_address);
 
   G_OBJECT_CLASS (klass)->finalize = gst_gl_context_finalize;
 
@@ -256,7 +298,7 @@ _init_debug (void)
   if (g_once_init_enter (&_init)) {
     GST_DEBUG_CATEGORY_INIT (gst_gl_context_debug, "glcontext", 0,
         "glcontext element");
-    GST_DEBUG_CATEGORY_GET (gst_performance, "GST_PERFORMANCE");
+    GST_DEBUG_CATEGORY_INIT (gst_gl_debug, "gldebug", 0, "OpenGL Debugging");
     g_once_init_leave (&_init, 1);
   }
 }
@@ -283,25 +325,24 @@ gst_gl_context_new (GstGLDisplay * display)
   GST_INFO ("creating a context for display %" GST_PTR_FORMAT
       ", user choice:%s", display, user_choice);
 #if GST_GL_HAVE_PLATFORM_CGL
-  if (!context && (!user_choice || g_strstr_len (user_choice, 5, "cgl")))
-    context = GST_GL_CONTEXT (gst_gl_context_cocoa_new ());
+  if (!context && (!user_choice || g_strstr_len (user_choice, 3, "cgl")))
+    context = GST_GL_CONTEXT (gst_gl_context_cocoa_new (display));
 #endif
 #if GST_GL_HAVE_PLATFORM_GLX
   if (!context && (!user_choice || g_strstr_len (user_choice, 3, "glx")))
-    context = GST_GL_CONTEXT (gst_gl_context_glx_new ());
+    context = GST_GL_CONTEXT (gst_gl_context_glx_new (display));
 #endif
 #if GST_GL_HAVE_PLATFORM_EGL
-  if (!context && (!user_choice || g_strstr_len (user_choice, 7, "egl")))
-    context = GST_GL_CONTEXT (gst_gl_context_egl_new ());
+  if (!context && (!user_choice || g_strstr_len (user_choice, 3, "egl")))
+    context = GST_GL_CONTEXT (gst_gl_context_egl_new (display));
 #endif
 #if GST_GL_HAVE_PLATFORM_WGL
-  if (!context && (!user_choice || g_strstr_len (user_choice, 3, "wgl"))) {
-    context = GST_GL_CONTEXT (gst_gl_context_wgl_new ());
-  }
+  if (!context && (!user_choice || g_strstr_len (user_choice, 3, "wgl")))
+    context = GST_GL_CONTEXT (gst_gl_context_wgl_new (display));
 #endif
 #if GST_GL_HAVE_PLATFORM_EAGL
-  if (!context && (!user_choice || g_strstr_len (user_choice, 5, "eagl")))
-    context = GST_GL_CONTEXT (gst_gl_context_eagl_new ());
+  if (!context && (!user_choice || g_strstr_len (user_choice, 4, "eagl")))
+    context = GST_GL_CONTEXT (gst_gl_context_eagl_new (display));
 #endif
 
   if (!context) {
@@ -312,7 +353,7 @@ gst_gl_context_new (GstGLDisplay * display)
     return NULL;
   }
 
-  context->priv->display = gst_object_ref (display);
+  context->display = gst_object_ref (display);
 
   GST_DEBUG_OBJECT (context,
       "Done creating context for display %" GST_PTR_FORMAT " (user_choice:%s)",
@@ -360,7 +401,8 @@ gst_gl_context_new_wrapped (GstGLDisplay * display, guintptr handle,
 
   context = (GstGLContext *) context_wrap;
 
-  context->priv->display = gst_object_ref (display);
+  context->display = gst_object_ref (display);
+  context->priv->sharegroup = _context_share_group_new ();
   context_wrap->handle = handle;
   context_wrap->platform = context_type;
   context_wrap->available_apis = available_apis;
@@ -383,7 +425,7 @@ gst_gl_context_new_wrapped (GstGLDisplay * display, guintptr handle,
   if (context_type == GST_GL_PLATFORM_CGL) {
     context_class->get_current_context =
         gst_gl_context_cocoa_get_current_context;
-    context_class->get_proc_address = _default_get_proc_address;
+    context_class->get_proc_address = gst_gl_context_default_get_proc_address;
   }
 #endif
 #if GST_GL_HAVE_PLATFORM_WGL
@@ -396,7 +438,7 @@ gst_gl_context_new_wrapped (GstGLDisplay * display, guintptr handle,
   if (context_type == GST_GL_PLATFORM_EAGL) {
     context_class->get_current_context =
         gst_gl_context_eagl_get_current_context;
-    context_class->get_proc_address = _default_get_proc_address;
+    context_class->get_proc_address = gst_gl_context_default_get_proc_address;
   }
 #endif
 
@@ -451,8 +493,35 @@ gst_gl_context_get_current_gl_context (GstGLPlatform context_type)
   return handle;
 }
 
+gpointer
+gst_gl_context_get_proc_address_with_platform (GstGLPlatform context_type,
+    GstGLAPI gl_api, const gchar * name)
+{
+  gpointer ret = NULL;
+
+#if GST_GL_HAVE_PLATFORM_GLX
+  if (!ret && (context_type & GST_GL_PLATFORM_GLX) != 0)
+    ret = gst_gl_context_glx_get_proc_address (gl_api, name);
+#endif
+#if GST_GL_HAVE_PLATFORM_EGL
+  if (!ret && (context_type & GST_GL_PLATFORM_EGL) != 0)
+    ret = gst_gl_context_egl_get_proc_address (gl_api, name);
+#endif
+#if GST_GL_HAVE_PLATFORM_WGL
+  if (!ret && (context_type & GST_GL_PLATFORM_WGL) != 0)
+    ret = gst_gl_context_wgl_get_proc_address (gl_api, name);
+#endif
+  /* CGL and EAGL rely on the default impl */
+
+  if (!ret)
+    ret = gst_gl_context_default_get_proc_address (gl_api, name);
+
+  return ret;
+}
+
 /**
  * gst_gl_context_get_current_gl_api:
+ * @platform: the #GstGLPlatform to retrieve the API for
  * @major: (out): (allow-none): the major version
  * @minor: (out): (allow-none): the minor version
  *
@@ -465,11 +534,12 @@ gst_gl_context_get_current_gl_context (GstGLPlatform context_type)
  * Since: 1.6
  */
 GstGLAPI
-gst_gl_context_get_current_gl_api (guint * major, guint * minor)
+gst_gl_context_get_current_gl_api (GstGLPlatform platform, guint * major,
+    guint * minor)
 {
-  const GLubyte *(*GetString) (GLenum name);
+  const GLubyte *(GSTGLAPI * GetString) (GLenum name);
 #if GST_GL_HAVE_OPENGL
-  void (*GetIntegerv) (GLenum name, GLuint * n);
+  void (GSTGLAPI * GetIntegerv) (GLenum name, GLuint * n);
 #endif
   const gchar *version;
   gint maj, min, n;
@@ -479,10 +549,13 @@ gst_gl_context_get_current_gl_api (guint * major, guint * minor)
 
   while (ret != GST_GL_API_NONE) {
     /* FIXME: attempt to delve into the platform specific GetProcAddress */
-    GetString = gst_gl_context_default_get_proc_address (ret, "glGetString");
+    GetString =
+        gst_gl_context_get_proc_address_with_platform (platform, ret,
+        "glGetString");
 #if GST_GL_HAVE_OPENGL
     GetIntegerv =
-        gst_gl_context_default_get_proc_address (ret, "glGetIntegerv");
+        gst_gl_context_get_proc_address_with_platform (platform, ret,
+        "glGetIntegerv");
 #endif
     if (!GetString) {
       goto next;
@@ -565,29 +638,30 @@ gst_gl_context_finalize (GObject * object)
     gst_gl_window_set_draw_callback (context->window, NULL, NULL, NULL);
 
     if (context->priv->alive) {
-      g_mutex_lock (&context->priv->render_lock);
       GST_INFO_OBJECT (context, "send quit gl window loop");
       gst_gl_window_quit (context->window);
-      while (context->priv->alive) {
-        g_cond_wait (&context->priv->destroy_cond, &context->priv->render_lock);
+
+      GST_INFO_OBJECT (context, "joining gl thread");
+      g_mutex_lock (&context->priv->render_lock);
+      if (context->priv->alive) {
+        GThread *t = context->priv->gl_thread;
+        g_mutex_unlock (&context->priv->render_lock);
+        g_thread_join (t);
+      } else {
+        g_mutex_unlock (&context->priv->render_lock);
       }
-      g_mutex_unlock (&context->priv->render_lock);
-    }
-
-    gst_gl_window_set_close_callback (context->window, NULL, NULL, NULL);
-
-    if (context->priv->gl_thread) {
-      gpointer ret = g_thread_join (context->priv->gl_thread);
       GST_INFO_OBJECT (context, "gl thread joined");
-      if (ret != NULL)
-        GST_ERROR_OBJECT (context, "gl thread returned a non-null pointer");
       context->priv->gl_thread = NULL;
     }
 
+    gst_gl_window_set_close_callback (context->window, NULL, NULL, NULL);
     gst_object_unref (context->window);
   }
 
-  gst_object_unref (context->priv->display);
+  if (context->priv->sharegroup)
+    _context_share_group_unref (context->priv->sharegroup);
+
+  gst_object_unref (context->display);
 
   if (context->gl_vtable) {
     g_slice_free (GstGLFuncs, context->gl_vtable);
@@ -596,7 +670,6 @@ gst_gl_context_finalize (GObject * object)
 
   g_mutex_clear (&context->priv->render_lock);
 
-  g_cond_clear (&context->priv->destroy_cond);
   g_cond_clear (&context->priv->create_cond);
 
   g_free (context->priv->gl_exts);
@@ -626,7 +699,7 @@ gst_gl_context_activate (GstGLContext * context, gboolean activate)
   GstGLContextClass *context_class;
   gboolean result;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_val_if_fail (context_class->activate != NULL, FALSE);
 
@@ -687,21 +760,13 @@ gst_gl_context_get_gl_api (GstGLContext * context)
 {
   GstGLContextClass *context_class;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), GST_GL_API_NONE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), GST_GL_API_NONE);
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_val_if_fail (context_class->get_gl_api != NULL, GST_GL_API_NONE);
 
   return context_class->get_gl_api (context);
 }
 
-static gpointer
-_default_get_proc_address (GstGLContext * context, const gchar * name)
-{
-  GstGLAPI gl_api = gst_gl_context_get_gl_api (context);
-
-  return gst_gl_context_default_get_proc_address (gl_api, name);
-}
-
 /**
  * gst_gl_context_get_proc_address:
  * @context: a #GstGLContext
@@ -722,12 +787,14 @@ gst_gl_context_get_proc_address (GstGLContext * context, const gchar * name)
 {
   gpointer ret;
   GstGLContextClass *context_class;
+  GstGLAPI gl_api;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), NULL);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), NULL);
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_val_if_fail (context_class->get_proc_address != NULL, NULL);
 
-  ret = context_class->get_proc_address (context, name);
+  gl_api = gst_gl_context_get_gl_api (context);
+  ret = context_class->get_proc_address (gl_api, name);
 
   return ret;
 }
@@ -778,7 +845,7 @@ gst_gl_context_default_get_proc_address (GstGLAPI gl_api, const gchar * name)
 gboolean
 gst_gl_context_set_window (GstGLContext * context, GstGLWindow * window)
 {
-  g_return_val_if_fail (!GST_GL_IS_WRAPPED_CONTEXT (context), FALSE);
+  g_return_val_if_fail (!GST_IS_GL_WRAPPED_CONTEXT (context), FALSE);
 
   GST_DEBUG_OBJECT (context, "window:%" GST_PTR_FORMAT, window);
 
@@ -812,9 +879,9 @@ gst_gl_context_set_window (GstGLContext * context, GstGLWindow * window)
 GstGLWindow *
 gst_gl_context_get_window (GstGLContext * context)
 {
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), NULL);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), NULL);
 
-  if (GST_GL_IS_WRAPPED_CONTEXT (context)) {
+  if (GST_IS_GL_WRAPPED_CONTEXT (context)) {
     GST_WARNING_OBJECT (context, "context is not toplevel, returning NULL");
     return NULL;
   }
@@ -824,53 +891,13 @@ gst_gl_context_get_window (GstGLContext * context)
   return gst_object_ref (context->window);
 }
 
-static gboolean
-_share_group_descendant (GstGLContext * context, GstGLContext * other_context,
-    GstGLContext ** root)
-{
-  GstGLContext *next = gst_object_ref (context);
-  GstGLContext *prev = NULL;
-
-  /* given a context tree where --> means "has other gl context":
-   *
-   * a-->b-->c-->d
-   *    /   /
-   *   e   /
-   *      /
-   * f-->g
-   *
-   * return TRUE if @other_context is a descendant of @context
-   *
-   * e.g. [a, b], [f, d], [e, c] are all descendants
-   * but [b, a], [d, f], [e, f] are not descendants.  Provide the root node (d)
-   * so that we can check if two chains end up at the end with the same
-   * GstGLContext
-   */
-
-  while (next != NULL) {
-    if (next == other_context) {
-      gst_object_unref (next);
-      if (root)
-        *root = NULL;
-      return TRUE;
-    }
-
-    prev = next;
-    next = g_weak_ref_get (&next->priv->other_context_ref);
-    gst_object_unref (prev);
-  }
-
-  if (root != NULL)
-    *root = prev;
-
-  return FALSE;
-}
-
 /**
  * gst_gl_context_can_share:
  * @context: a #GstGLContext
  * @other_context: another #GstGLContext
  *
+ * Note: This will always fail for two wrapped #GstGLContext's
+ *
  * Returns: whether @context and @other_context are able to share OpenGL
  *      resources.
  *
@@ -879,16 +906,12 @@ _share_group_descendant (GstGLContext * context, GstGLContext * other_context,
 gboolean
 gst_gl_context_can_share (GstGLContext * context, GstGLContext * other_context)
 {
-  GstGLContext *root1, *root2;
-
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (other_context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (other_context), FALSE);
 
   /* check if the contexts are descendants or the root nodes are the same */
-  return context == other_context
-      || _share_group_descendant (context, other_context, &root1)
-      || _share_group_descendant (other_context, context, &root2)
-      || ((root1 != NULL || root2 != NULL) && root1 == root2);
+  return context->priv->sharegroup != NULL
+      && context->priv->sharegroup == other_context->priv->sharegroup;
 }
 
 /**
@@ -916,8 +939,8 @@ gst_gl_context_create (GstGLContext * context,
 {
   gboolean alive = FALSE;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
-  g_return_val_if_fail (!GST_GL_IS_WRAPPED_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
+  g_return_val_if_fail (!GST_IS_GL_WRAPPED_CONTEXT (context), FALSE);
 
   GST_DEBUG_OBJECT (context, " other_context:%" GST_PTR_FORMAT, other_context);
 
@@ -928,13 +951,17 @@ gst_gl_context_create (GstGLContext * context,
   if (!context->priv->created) {
     g_weak_ref_set (&context->priv->other_context_ref, other_context);
     context->priv->error = error;
+    if (other_context == NULL)
+      context->priv->sharegroup = _context_share_group_new ();
+    else
+      context->priv->sharegroup =
+          _context_share_group_ref (other_context->priv->sharegroup);
 
     context->priv->gl_thread = g_thread_new ("gstglcontext",
         (GThreadFunc) gst_gl_context_create_thread, context);
 
-    g_cond_wait (&context->priv->create_cond, &context->priv->render_lock);
-
-    context->priv->created = TRUE;
+    while (!context->priv->created)
+      g_cond_wait (&context->priv->create_cond, &context->priv->render_lock);
 
     GST_INFO_OBJECT (context, "gl thread created");
   }
@@ -946,154 +973,6 @@ gst_gl_context_create (GstGLContext * context,
   return alive;
 }
 
-#ifndef GL_DEBUG_TYPE_ERROR
-#define GL_DEBUG_TYPE_ERROR 0x824C
-#endif
-#ifndef GL_DEBUG_TYPE_DEPRECATED_BEHAVIOUR
-#define GL_DEBUG_TYPE_DEPRECATED_BEHAVIOUR 0x824D
-#endif
-#ifndef GL_DEBUG_TYPE_UNDEFINED_BEHAVIOUR
-#define GL_DEBUG_TYPE_UNDEFINED_BEHAVIOUR 0x824E
-#endif
-#ifndef GL_DEBUG_TYPE_PORTABILITY
-#define GL_DEBUG_TYPE_PORTABILITY 0x824F
-#endif
-#ifndef GL_DEBUG_TYPE_PERFORMANCE
-#define GL_DEBUG_TYPE_PERFORMANCE 0x8250
-#endif
-#ifndef GL_DEBUG_TYPE_MARKER
-#define GL_DEBUG_TYPE_MARKER 0x8268
-#endif
-#ifndef GL_DEBUG_TYPE_OTHER
-#define GL_DEBUG_TYPE_OTHER 0x8251
-#endif
-
-#ifndef GL_DEBUG_SEVERITY_HIGH
-#define GL_DEBUG_SEVERITY_HIGH 0x9146
-#endif
-#ifndef GL_DEBUG_SEVERITY_MEDIUM
-#define GL_DEBUG_SEVERITY_MEDIUM 0x9147
-#endif
-#ifndef GL_DEBUG_SEVERITY_LOW
-#define GL_DEBUG_SEVERITY_LOW 0x9148
-#endif
-#ifndef GL_DEBUG_SEVERITY_NOTIFICATION
-#define GL_DEBUG_SEVERITY_NOTIFICATION 0x826B
-#endif
-
-#ifndef GL_DEBUG_SOURCE_API
-#define GL_DEBUG_SOURCE_API 0x8246
-#endif
-#ifndef GL_DEBUG_SOURCE_WINDOW_SYSTEM
-#define GL_DEBUG_SOURCE_WINDOW_SYSTEM 0x8247
-#endif
-#ifndef GL_DEBUG_SOURCE_SHADER_COMPILER
-#define GL_DEBUG_SOURCE_SHADER_COMPILER 0x8248
-#endif
-#ifndef GL_DEBUG_SOURCE_THIRD_PARTY
-#define GL_DEBUG_SOURCE_THIRD_PARTY 0x8249
-#endif
-#ifndef GL_DEBUG_SOURCE_APPLICATION
-#define GL_DEBUG_SOURCE_APPLICATION 0x824A
-#endif
-#ifndef GL_DEBUG_SOURCE_OTHER
-#define GL_DEBUG_SOURCE_OTHER 0x824B
-#endif
-
-#if !defined(GST_DISABLE_GST_DEBUG)
-static inline const gchar *
-_debug_severity_to_string (GLenum severity)
-{
-  switch (severity) {
-    case GL_DEBUG_SEVERITY_HIGH:
-      return "high";
-    case GL_DEBUG_SEVERITY_MEDIUM:
-      return "medium";
-    case GL_DEBUG_SEVERITY_LOW:
-      return "low";
-    case GL_DEBUG_SEVERITY_NOTIFICATION:
-      return "notification";
-    default:
-      return "invalid";
-  }
-}
-
-static inline const gchar *
-_debug_source_to_string (GLenum source)
-{
-  switch (source) {
-    case GL_DEBUG_SOURCE_API:
-      return "API";
-    case GL_DEBUG_SOURCE_WINDOW_SYSTEM:
-      return "winsys";
-    case GL_DEBUG_SOURCE_SHADER_COMPILER:
-      return "shader compiler";
-    case GL_DEBUG_SOURCE_THIRD_PARTY:
-      return "third party";
-    case GL_DEBUG_SOURCE_APPLICATION:
-      return "application";
-    case GL_DEBUG_SOURCE_OTHER:
-      return "other";
-    default:
-      return "invalid";
-  }
-}
-
-static inline const gchar *
-_debug_type_to_string (GLenum type)
-{
-  switch (type) {
-    case GL_DEBUG_TYPE_ERROR:
-      return "error";
-    case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOUR:
-      return "deprecated";
-    case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOUR:
-      return "undefined";
-    case GL_DEBUG_TYPE_PORTABILITY:
-      return "portability";
-    case GL_DEBUG_TYPE_PERFORMANCE:
-      return "performance";
-    case GL_DEBUG_TYPE_MARKER:
-      return "debug marker";
-    case GL_DEBUG_TYPE_OTHER:
-      return "other";
-    default:
-      return "invalid";
-  }
-}
-
-static void GSTGLAPI
-_gst_gl_debug_callback (GLenum source, GLenum type, GLuint id, GLenum severity,
-    GLsizei length, const gchar * message, gpointer user_data)
-{
-  GstGLContext *context = user_data;
-  const gchar *severity_str = _debug_severity_to_string (severity);
-  const gchar *source_str = _debug_source_to_string (source);
-  const gchar *type_str = _debug_type_to_string (type);
-
-  switch (type) {
-    case GL_DEBUG_TYPE_ERROR:
-    case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOUR:
-      GST_ERROR_OBJECT (context, "%s: GL %s from %s id:%u, %s", severity_str,
-          type_str, source_str, id, message);
-      break;
-    case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOUR:
-    case GL_DEBUG_TYPE_PORTABILITY:
-      GST_FIXME_OBJECT (context, "%s: GL %s from %s id:%u, %s", severity_str,
-          type_str, source_str, id, message);
-      break;
-    case GL_DEBUG_TYPE_PERFORMANCE:
-      GST_CAT_DEBUG_OBJECT (gst_performance, context, "%s: GL %s from %s id:%u,"
-          " %s", severity_str, type_str, source_str, id, message);
-      break;
-    default:
-      GST_DEBUG_OBJECT (context, "%s: GL %s from %s id:%u, %s", severity_str,
-          type_str, source_str, id, message);
-      break;
-  }
-}
-#endif
-
 static gboolean
 _create_context_info (GstGLContext * context, GstGLAPI gl_api, gint * gl_major,
     gint * gl_minor, GError ** error)
@@ -1105,13 +984,18 @@ _create_context_info (GstGLContext * context, GstGLAPI gl_api, gint * gl_major,
 
   gl = context->gl_vtable;
 
-  if (!gl->GetString || !gl->GetString (GL_VERSION)
-      || !gl->GetString (GL_SHADING_LANGUAGE_VERSION)) {
+  if (!gl->GetString || !gl->GetString (GL_VERSION)) {
     g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
         "glGetString not defined or returned invalid value");
     return FALSE;
   }
 
+  if (!gl->GetString (GL_SHADING_LANGUAGE_VERSION)) {
+    g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
+        "No GL shader support available");
+    return FALSE;
+  }
+
   GST_INFO_OBJECT (context, "GL_VERSION: %s",
       GST_STR_NULL ((const gchar *) gl->GetString (GL_VERSION)));
   GST_INFO_OBJECT (context, "GL_SHADING_LANGUAGE_VERSION: %s",
@@ -1172,6 +1056,9 @@ _compiled_api (void)
 static void
 _unlock_create_thread (GstGLContext * context)
 {
+  context->priv->created = TRUE;
+  GST_INFO_OBJECT (context, "gl thread running");
+  g_cond_signal (&context->priv->create_cond);
   g_mutex_unlock (&context->priv->render_lock);
 }
 
@@ -1181,7 +1068,8 @@ _build_extension_string (GstGLContext * context)
   const GstGLFuncs *gl = context->gl_vtable;
   GString *ext_g_str = g_string_sized_new (1024);
   const gchar *ext_const_c_str = NULL;
-  int i, n;
+  GLint i = 0;
+  GLint n = 0;
 
   gl->GetIntegerv (GL_NUM_EXTENSIONS, &n);
 
@@ -1201,7 +1089,6 @@ gst_gl_context_create_thread (GstGLContext * context)
 {
   GstGLContextClass *context_class;
   GstGLWindowClass *window_class;
-  GstGLFuncs *gl;
   GstGLAPI compiled_api, user_api, gl_api, display_api;
   gchar *api_string;
   gchar *compiled_api_s;
@@ -1221,7 +1108,7 @@ gst_gl_context_create_thread (GstGLContext * context)
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   window_class = GST_GL_WINDOW_GET_CLASS (context->window);
 
-  display_api = gst_gl_display_get_gl_api_unlocked (context->priv->display);
+  display_api = gst_gl_display_get_gl_api_unlocked (context->display);
   if (display_api == GST_GL_API_NONE) {
     g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_WRONG_API,
         "Cannot create context with satisfying requested apis "
@@ -1237,7 +1124,6 @@ gst_gl_context_create_thread (GstGLContext * context)
     }
   }
 
-  gl = context->gl_vtable;
   compiled_api = _compiled_api ();
   compiled_api_s = gst_gl_api_to_string (compiled_api);
 
@@ -1260,7 +1146,7 @@ gst_gl_context_create_thread (GstGLContext * context)
 
   if (context_class->choose_format &&
       !context_class->choose_format (context, error)) {
-    GST_WARNING ("Failed to choose format");
+    GST_WARNING_OBJECT (context, "Failed to choose format");
     g_assert (error == NULL || *error != NULL);
     g_free (compiled_api_s);
     g_free (user_api_s);
@@ -1325,15 +1211,9 @@ gst_gl_context_create_thread (GstGLContext * context)
 
   context->priv->alive = TRUE;
 
-  if (gl->DebugMessageCallback) {
 #if !defined(GST_DISABLE_GST_DEBUG)
-    GST_INFO_OBJECT (context, "Enabling GL context debugging");
-    /* enable them all */
-    gl->DebugMessageControl (GL_DONT_CARE, GL_DONT_CARE, GL_DONT_CARE, 0, 0,
-        GL_TRUE);
-    gl->DebugMessageCallback (_gst_gl_debug_callback, context);
+  _gst_gl_debug_enable (context);
 #endif
-  }
 
   if (other_context) {
     GST_DEBUG_OBJECT (context, "Unreffing other_context %" GST_PTR_FORMAT,
@@ -1341,9 +1221,8 @@ gst_gl_context_create_thread (GstGLContext * context)
     gst_object_unref (other_context);
   }
 
-  g_cond_signal (&context->priv->create_cond);
-
-//  g_mutex_unlock (&context->priv->render_lock);
+  /* unlocking of the render_lock happens when the
+   * context's loop is running from inside that loop */
   gst_gl_window_send_message_async (context->window,
       (GstGLWindowCB) _unlock_create_thread, context, NULL);
 
@@ -1352,7 +1231,6 @@ gst_gl_context_create_thread (GstGLContext * context)
   GST_INFO_OBJECT (context, "loop exited");
 
   g_mutex_lock (&context->priv->render_lock);
-
   context->priv->alive = FALSE;
 
   gst_gl_context_activate (context, FALSE);
@@ -1368,8 +1246,7 @@ gst_gl_context_create_thread (GstGLContext * context)
     window_class->close (context->window);
   }
 
-  g_cond_signal (&context->priv->destroy_cond);
-
+  context->priv->created = FALSE;
   g_mutex_unlock (&context->priv->render_lock);
 
   return NULL;
@@ -1379,6 +1256,9 @@ failure:
     if (other_context)
       gst_object_unref (other_context);
 
+    /* A context that fails to be created is considered created but not alive
+     * and will never be able to be alive as creation can't happen */
+    context->priv->created = TRUE;
     g_cond_signal (&context->priv->create_cond);
     g_mutex_unlock (&context->priv->render_lock);
     return NULL;
@@ -1401,7 +1281,7 @@ gst_gl_context_destroy (GstGLContext * context)
 {
   GstGLContextClass *context_class;
 
-  g_return_if_fail (GST_GL_IS_CONTEXT (context));
+  g_return_if_fail (GST_IS_GL_CONTEXT (context));
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_if_fail (context_class->destroy_context != NULL);
 
@@ -1427,7 +1307,7 @@ gst_gl_context_fill_info (GstGLContext * context, GError ** error)
   GstGLAPI gl_api;
   gboolean ret;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
   g_return_val_if_fail (context->priv->active_thread == g_thread_self (),
       FALSE);
 
@@ -1485,6 +1365,18 @@ gst_gl_context_fill_info (GstGLContext * context, GError ** error)
     goto failure;
   }
 
+  /* Does not implement OES_vertex_array_object properly, see
+   * https://bugzilla.gnome.org/show_bug.cgi?id=750185 */
+  if (g_strcmp0 ((const gchar *) gl->GetString (GL_VENDOR),
+          "Imagination Technologies") == 0
+      && g_strcmp0 ((const gchar *) gl->GetString (GL_RENDERER),
+          "PowerVR SGX 544MP") == 0) {
+    gl->GenVertexArrays = NULL;
+    gl->DeleteVertexArrays = NULL;
+    gl->BindVertexArray = NULL;
+    gl->IsVertexArray = NULL;
+  }
+
   return TRUE;
 
 failure:
@@ -1507,7 +1399,7 @@ gst_gl_context_get_gl_context (GstGLContext * context)
   GstGLContextClass *context_class;
   guintptr result;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), 0);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), 0);
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_val_if_fail (context_class->get_gl_context != NULL, 0);
 
@@ -1531,7 +1423,7 @@ gst_gl_context_get_gl_platform (GstGLContext * context)
 {
   GstGLContextClass *context_class;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), 0);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), 0);
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
   g_return_val_if_fail (context_class->get_gl_platform != NULL, 0);
 
@@ -1549,28 +1441,9 @@ gst_gl_context_get_gl_platform (GstGLContext * context)
 GstGLDisplay *
 gst_gl_context_get_display (GstGLContext * context)
 {
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), NULL);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), NULL);
 
-  return gst_object_ref (context->priv->display);
-}
-
-/**
- * gst_gl_context_set_display:
- * @context: a #GstGLContext:
- * @display: a #GstGLDisplay:
- *
- * Ref @display and unref previous display if exists.
- *
- * Since: 1.6
- */
-void
-gst_gl_context_set_display (GstGLContext * context, GstGLDisplay * display)
-{
-  g_return_if_fail (GST_GL_IS_CONTEXT (context));
-  g_return_if_fail (GST_IS_GL_DISPLAY (display));
-
-  gst_object_replace ((GstObject **) & context->priv->display,
-      (GstObject *) display);
+  return gst_object_ref (context->display);
 }
 
 typedef struct
@@ -1583,7 +1456,8 @@ typedef struct
 static void
 _gst_gl_context_thread_run_generic (RunGenericData * data)
 {
-  GST_TRACE ("running function:%p data:%p", data->func, data->data);
+  GST_TRACE_OBJECT (data->context, "running function:%p data:%p", data->func,
+      data->data);
 
   data->func (data->context, data->data);
 }
@@ -1607,11 +1481,13 @@ gst_gl_context_thread_add (GstGLContext * context,
   GstGLWindow *window;
   RunGenericData rdata;
 
-  g_return_if_fail (GST_GL_IS_CONTEXT (context));
+  g_return_if_fail (GST_IS_GL_CONTEXT (context));
   g_return_if_fail (func != NULL);
 
-  if (GST_GL_IS_WRAPPED_CONTEXT (context)) {
+  if (GST_IS_GL_WRAPPED_CONTEXT (context))
     g_return_if_fail (context->priv->active_thread == g_thread_self ());
+
+  if (context->priv->active_thread == g_thread_self ()) {
     func (context, data);
     return;
   }
@@ -1643,8 +1519,8 @@ gst_gl_context_thread_add (GstGLContext * context,
 void
 gst_gl_context_get_gl_version (GstGLContext * context, gint * maj, gint * min)
 {
-  g_return_if_fail (GST_GL_IS_CONTEXT (context));
-  g_return_if_fail (maj != NULL && min != NULL);
+  g_return_if_fail (GST_IS_GL_CONTEXT (context));
+  g_return_if_fail (!(maj == NULL && min == NULL));
 
   if (maj)
     *maj = context->priv->gl_major;
@@ -1669,7 +1545,7 @@ gboolean
 gst_gl_context_check_gl_version (GstGLContext * context, GstGLAPI api,
     gint maj, gint min)
 {
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
 
   if (maj > context->priv->gl_major)
     return FALSE;
@@ -1703,7 +1579,7 @@ gst_gl_context_check_feature (GstGLContext * context, const gchar * feature)
 {
   GstGLContextClass *context_class;
 
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
   g_return_val_if_fail (feature != NULL, FALSE);
 
   context_class = GST_GL_CONTEXT_GET_CLASS (context);
@@ -1732,6 +1608,50 @@ gst_gl_context_get_current (void)
   return g_private_get (&current_context_key);
 }
 
+/**
+ * gst_gl_context_is_shared:
+ * @context: a #GstGLContext
+ *
+ * Returns: Whether the #GstGLContext has been shared with another #GstGLContext
+ *
+ * Since: 1.8
+ */
+gboolean
+gst_gl_context_is_shared (GstGLContext * context)
+{
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
+  if (GST_IS_GL_WRAPPED_CONTEXT (context))
+    g_return_val_if_fail (context->priv->active_thread, FALSE);
+  else
+    g_return_val_if_fail (context->priv->alive, FALSE);
+
+  return _context_share_group_is_shared (context->priv->sharegroup);
+}
+
+/**
+ * gst_gl_context_set_shared_with:
+ * @context: a wrapped #GstGLContext
+ * @share: another #GstGLContext
+ *
+ * Will internally set @context as shared with @share
+ *
+ * Since: 1.8
+ */
+void
+gst_gl_context_set_shared_with (GstGLContext * context, GstGLContext * share)
+{
+  g_return_if_fail (GST_IS_GL_CONTEXT (context));
+  g_return_if_fail (GST_IS_GL_CONTEXT (share));
+  g_return_if_fail (!gst_gl_context_is_shared (context));
+  /* XXX: may be a little too strict */
+  g_return_if_fail (GST_IS_GL_WRAPPED_CONTEXT (context));
+
+  if (context->priv->sharegroup)
+    _context_share_group_unref (context->priv->sharegroup);
+  context->priv->sharegroup =
+      _context_share_group_ref (share->priv->sharegroup);
+}
+
 static GstGLAPI
 gst_gl_wrapped_context_get_gl_api (GstGLContext * context)
 {
