@@ -67,7 +67,8 @@
 #endif
 #if GST_GL_HAVE_PLATFORM_EGL
 #include <gst/gl/egl/gstgldisplay_egl.h>
-#include <gst/gl/egl/gsteglimagememory.h>
+#include <gst/gl/egl/gsteglimage.h>
+#include <gst/gl/egl/gstglmemoryegl.h>
 #endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_context);
@@ -84,6 +85,15 @@ G_DEFINE_TYPE_WITH_CODE (GstGLDisplay, gst_gl_display, GST_TYPE_OBJECT,
 #define GST_GL_DISPLAY_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_DISPLAY, GstGLDisplayPrivate))
 
+enum
+{
+  SIGNAL_0,
+  CREATE_CONTEXT,
+  LAST_SIGNAL
+};
+
+static guint gst_gl_display_signals[LAST_SIGNAL] = { 0 };
+
 static void gst_gl_display_finalize (GObject * object);
 static guintptr gst_gl_display_default_get_handle (GstGLDisplay * display);
 
@@ -99,6 +109,22 @@ gst_gl_display_class_init (GstGLDisplayClass * klass)
 {
   g_type_class_add_private (klass, sizeof (GstGLDisplayPrivate));
 
+  /**
+   * GstGLDisplay::create-context:
+   * @object: the #GstGLDisplay
+   * @context: other context to share resources with.
+   *
+   * Overrides the @GstGLContext creation mechanism.
+   * It can be called in any thread and it is emitted with
+   * display's object lock held.
+   *
+   * Returns: the new context.
+   */
+  gst_gl_display_signals[CREATE_CONTEXT] =
+      g_signal_new ("create-context", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic,
+      GST_GL_TYPE_CONTEXT, 1, GST_GL_TYPE_CONTEXT);
+
   klass->get_handle = gst_gl_display_default_get_handle;
 
   G_OBJECT_CLASS (klass)->finalize = gst_gl_display_finalize;
@@ -114,10 +140,12 @@ gst_gl_display_init (GstGLDisplay * display)
 
   GST_TRACE ("init %p", display);
 
-  gst_gl_memory_init ();
+  gst_gl_buffer_init_once ();
+  gst_gl_memory_pbo_init_once ();
+  gst_gl_renderbuffer_init_once ();
 
 #if GST_GL_HAVE_PLATFORM_EGL
-  gst_egl_image_memory_init ();
+  gst_gl_memory_egl_init_once ();
 #endif
 }
 
@@ -185,8 +213,7 @@ gst_gl_display_new (void)
     display = GST_GL_DISPLAY (gst_gl_display_egl_new ());
 #endif
   if (!display) {
-    /* subclass returned a NULL window */
-    GST_WARNING ("Could not create display. user specified %s "
+    GST_INFO ("Could not create platform/winsys display. user specified %s "
         "(platform: %s), creating dummy",
         GST_STR_NULL (user_choice), GST_STR_NULL (platform_choice));
 
@@ -340,6 +367,58 @@ gst_context_get_gl_display (GstContext * context, GstGLDisplay ** display)
   return ret;
 }
 
+/**
+ * gst_gl_display_create_context:
+ * @display: a #GstGLDisplay
+ * @other_context: other #GstGLContext to share resources with.
+ * @p_context: resulting #GstGLContext
+ * @error: resulting #GError
+ *
+ * It requires the display's object lock to be held.
+ *
+ * Returns: whether a new context could be created.
+ *
+ * Since: 1.6
+ */
+gboolean
+gst_gl_display_create_context (GstGLDisplay * display,
+    GstGLContext * other_context, GstGLContext ** p_context, GError ** error)
+{
+  GstGLContext *context = NULL;
+  gboolean ret = FALSE;
+
+  g_return_val_if_fail (display != NULL, FALSE);
+  g_return_val_if_fail (p_context != NULL, FALSE);
+  g_return_val_if_fail (error != NULL, FALSE);
+  g_return_val_if_fail (*error == NULL, FALSE);
+
+  g_signal_emit (display, gst_gl_display_signals[CREATE_CONTEXT], 0,
+      other_context, &context);
+
+  if (context) {
+    *p_context = context;
+    return TRUE;
+  }
+
+  context = gst_gl_context_new (display);
+  if (!context) {
+    g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
+        "Failed to create GL context");
+    return FALSE;
+  }
+
+  GST_DEBUG_OBJECT (display,
+      "creating context %" GST_PTR_FORMAT " from other context %"
+      GST_PTR_FORMAT, context, other_context);
+
+  ret = gst_gl_context_create (context, other_context, error);
+
+  if (ret)
+    *p_context = context;
+
+  return ret;
+}
+
 static GstGLContext *
 _get_gl_context_for_thread_unlocked (GstGLDisplay * display, GThread * thread)
 {
@@ -354,13 +433,20 @@ _get_gl_context_for_thread_unlocked (GstGLDisplay * display, GThread * thread)
     if (!context) {
       /* remove dead contexts */
       g_weak_ref_clear (l->data);
+      g_free (l->data);
       display->priv->contexts = g_list_delete_link (display->priv->contexts, l);
       l = prev ? prev->next : display->priv->contexts;
       continue;
     }
 
+    if (thread == NULL) {
+      GST_DEBUG_OBJECT (display, "Returning GL context %" GST_PTR_FORMAT " for "
+          "NULL thread", context);
+      return context;
+    }
+
     context_thread = gst_gl_context_get_thread (context);
-    if (thread != NULL && thread == context_thread) {
+    if (thread != context_thread) {
       g_thread_unref (context_thread);
       gst_object_unref (context);
       prev = l;
@@ -370,9 +456,13 @@ _get_gl_context_for_thread_unlocked (GstGLDisplay * display, GThread * thread)
 
     if (context_thread)
       g_thread_unref (context_thread);
+
+    GST_DEBUG_OBJECT (display, "Returning GL context %" GST_PTR_FORMAT " for "
+        "thread %p", context, thread);
     return context;
   }
 
+  GST_DEBUG_OBJECT (display, "No GL context for thread %p", thread);
   return NULL;
 }
 
@@ -419,7 +509,7 @@ _check_collision (GstGLContext * context, GstGLContext * collision)
     goto out;
   }
 
-  if (collision == context) {
+  if (thread == collision_thread) {
     ret = TRUE;
     goto out;
   }
@@ -455,7 +545,7 @@ gst_gl_display_add_context (GstGLDisplay * display, GstGLContext * context)
   GWeakRef *ref;
 
   g_return_val_if_fail (GST_IS_GL_DISPLAY (display), FALSE);
-  g_return_val_if_fail (GST_GL_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (GST_IS_GL_CONTEXT (context), FALSE);
 
   context_display = gst_gl_context_get_display (context);
   g_assert (context_display == display);
@@ -465,7 +555,18 @@ gst_gl_display_add_context (GstGLDisplay * display, GstGLContext * context)
   if (thread) {
     collision = _get_gl_context_for_thread_unlocked (display, thread);
     g_thread_unref (thread);
+
+    /* adding the same context is a no-op */
+    if (context == collision) {
+      GST_LOG_OBJECT (display, "Attempting to add the same GL context %"
+          GST_PTR_FORMAT ". Ignoring", context);
+      ret = TRUE;
+      goto out;
+    }
+
     if (_check_collision (context, collision)) {
+      GST_DEBUG_OBJECT (display, "Collision detected adding GL context "
+          "%" GST_PTR_FORMAT, context);
       ret = FALSE;
       goto out;
     }
@@ -474,6 +575,7 @@ gst_gl_display_add_context (GstGLDisplay * display, GstGLContext * context)
   ref = g_new0 (GWeakRef, 1);
   g_weak_ref_init (ref, context);
 
+  GST_DEBUG_OBJECT (display, "Adding GL context %" GST_PTR_FORMAT, context);
   display->priv->contexts = g_list_prepend (display->priv->contexts, ref);
 
 out:
