@@ -31,17 +31,14 @@
 
 /**
  * SECTION:gstglrenderbuffer
- * @title: GstGLRenderBuffer
  * @short_description: memory subclass for GL renderbuffer objects
  * @see_also: #GstMemory, #GstAllocator
  *
  * GstGLRenderbuffer is a #GstGLBaseMemory subclass providing support for
- * OpenGL renderbuffers.
+ * OpenGL renderbuffers.  
  *
  * #GstGLRenderbuffer is created or wrapped through gst_gl_base_memory_alloc()
  * with #GstGLRenderbufferAllocationParams.
- *
- * Since: 1.10
  */
 
 #define USING_OPENGL(context) (gst_gl_context_check_gl_version (context, GST_GL_API_OPENGL, 1, 0))
@@ -84,9 +81,9 @@ _gl_rbo_create (GstGLRenderbuffer * gl_mem, GError ** error)
     GLenum tex_format;
     GLenum renderbuffer_type;
 
-    tex_format = gl_mem->renderbuffer_format;
+    tex_format = gst_gl_format_from_gl_texture_type (gl_mem->renderbuffer_type);
     renderbuffer_type = GL_UNSIGNED_BYTE;
-    if (gl_mem->renderbuffer_format == GST_GL_RGB565)
+    if (gl_mem->renderbuffer_type == GST_VIDEO_GL_TEXTURE_TYPE_RGB16)
       renderbuffer_type = GL_UNSIGNED_SHORT_5_6_5;
 
     internal_format =
@@ -110,20 +107,12 @@ _gl_rbo_create (GstGLRenderbuffer * gl_mem, GError ** error)
 static void
 gst_gl_renderbuffer_init (GstGLRenderbuffer * mem, GstAllocator * allocator,
     GstMemory * parent, GstGLContext * context,
-    GstGLFormat renderbuffer_format, GstAllocationParams * params,
+    GstVideoGLTextureType renderbuffer_type, GstAllocationParams * params,
     guint width, guint height, gpointer user_data, GDestroyNotify notify)
 {
-  gsize size;
-  guint tex_type;
+  gsize size = gst_gl_texture_type_n_bytes (renderbuffer_type) * width * height;
 
-  tex_type = GL_UNSIGNED_BYTE;
-  if (renderbuffer_format == GST_GL_RGB565)
-    tex_type = GL_UNSIGNED_SHORT_5_6_5;
-  size =
-      gst_gl_format_type_n_bytes (renderbuffer_format,
-      tex_type) * width * height;
-
-  mem->renderbuffer_format = renderbuffer_format;
+  mem->renderbuffer_type = renderbuffer_type;
   mem->width = width;
   mem->height = height;
 
@@ -132,7 +121,7 @@ gst_gl_renderbuffer_init (GstGLRenderbuffer * mem, GstAllocator * allocator,
 
   GST_CAT_DEBUG (GST_CAT_GL_RENDERBUFFER, "new GL renderbuffer context:%"
       GST_PTR_FORMAT " memory:%p format:%u dimensions:%ux%u ", context, mem,
-      mem->renderbuffer_format, gst_gl_renderbuffer_get_width (mem),
+      mem->renderbuffer_type, gst_gl_renderbuffer_get_width (mem),
       gst_gl_renderbuffer_get_height (mem));
 }
 
@@ -193,7 +182,7 @@ _default_gl_rbo_alloc (GstGLRenderbufferAllocator * allocator,
   }
 
   gst_gl_renderbuffer_init (mem, GST_ALLOCATOR_CAST (allocator), NULL,
-      params->parent.context, params->renderbuffer_format,
+      params->parent.context, params->renderbuffer_type,
       params->parent.alloc_params, params->width, params->height,
       params->parent.user_data, params->parent.notify);
 
@@ -238,7 +227,7 @@ gst_gl_renderbuffer_allocator_init (GstGLRenderbufferAllocator * allocator)
  *
  * Returns: the configured width of @gl_mem
  *
- * Since: 1.10
+ * Since: 1.8
  */
 gint
 gst_gl_renderbuffer_get_width (GstGLRenderbuffer * gl_mem)
@@ -254,7 +243,7 @@ gst_gl_renderbuffer_get_width (GstGLRenderbuffer * gl_mem)
  *
  * Returns: the configured height of @gl_mem
  *
- * Since: 1.10
+ * Since: 1.8
  */
 gint
 gst_gl_renderbuffer_get_height (GstGLRenderbuffer * gl_mem)
@@ -265,19 +254,19 @@ gst_gl_renderbuffer_get_height (GstGLRenderbuffer * gl_mem)
 }
 
 /**
- * gst_gl_renderbuffer_get_format:
+ * gst_gl_renderbuffer_get_type:
  * @gl_mem: a #GstGLRenderbuffer
  *
- * Returns: the #GstGLFormat of @gl_mem
+ * Returns: the #GstVideoGLTextureType of @gl_mem
  *
- * Since: 1.12
+ * Since: 1.8
  */
-GstGLFormat
-gst_gl_renderbuffer_get_format (GstGLRenderbuffer * gl_mem)
+GstVideoGLTextureType
+gst_gl_renderbuffer_get_type (GstGLRenderbuffer * gl_mem)
 {
   g_return_val_if_fail (gst_is_gl_renderbuffer ((GstMemory *) gl_mem), 0);
 
-  return gl_mem->renderbuffer_format;
+  return gl_mem->renderbuffer_type;
 }
 
 /**
@@ -286,7 +275,7 @@ gst_gl_renderbuffer_get_format (GstGLRenderbuffer * gl_mem)
  *
  * Returns: the OpenGL renderbuffer handle of @gl_mem
  *
- * Since: 1.10
+ * Since: 1.8
  */
 guint
 gst_gl_renderbuffer_get_id (GstGLRenderbuffer * gl_mem)
@@ -302,7 +291,7 @@ gst_gl_renderbuffer_get_id (GstGLRenderbuffer * gl_mem)
  * Initializes the GL Base Texture allocator. It is safe to call this function
  * multiple times.  This must be called before any other GstGLRenderbuffer operation.
  *
- * Since: 1.10
+ * Since: 1.4
  */
 void
 gst_gl_renderbuffer_init_once (void)
@@ -330,7 +319,7 @@ gst_gl_renderbuffer_init_once (void)
 /**
  * gst_is_gl_renderbuffer:
  * @mem:a #GstMemory
- *
+ * 
  * Returns: whether the memory at @mem is a #GstGLRenderbuffer
  *
  * Since: 1.10
@@ -343,11 +332,6 @@ gst_is_gl_renderbuffer (GstMemory * mem)
       GST_TYPE_GL_RENDERBUFFER_ALLOCATOR);
 }
 
-G_DEFINE_BOXED_TYPE (GstGLRenderbufferAllocationParams,
-    gst_gl_renderbuffer_allocation_params,
-    (GBoxedCopyFunc) gst_gl_allocation_params_copy,
-    (GBoxedFreeFunc) gst_gl_allocation_params_free);
-
 static void
 _gst_gl_rb_alloc_params_free_data (GstGLRenderbufferAllocationParams * params)
 {
@@ -363,7 +347,7 @@ _gst_gl_rb_alloc_params_copy_data (GstGLRenderbufferAllocationParams * src_vid,
 
   gst_gl_allocation_params_copy_data (src, dest);
 
-  dest_vid->renderbuffer_format = src_vid->renderbuffer_format;
+  dest_vid->renderbuffer_type = src_vid->renderbuffer_type;
   dest_vid->width = src_vid->width;
   dest_vid->height = src_vid->height;
 }
@@ -374,7 +358,7 @@ static gboolean
     guint alloc_flags, GstGLAllocationParamsCopyFunc copy,
     GstGLAllocationParamsFreeFunc free, GstGLContext * context,
     GstAllocationParams * alloc_params, guint width, guint height,
-    GstGLFormat renderbuffer_format, gpointer wrapped_data,
+    GstVideoGLTextureType renderbuffer_type, gpointer wrapped_data,
     gpointer gl_handle, gpointer user_data, GDestroyNotify notify)
 {
   g_return_val_if_fail (params != NULL, FALSE);
@@ -389,7 +373,7 @@ static gboolean
           wrapped_data, gl_handle, user_data, notify))
     return FALSE;
 
-  params->renderbuffer_format = renderbuffer_format;
+  params->renderbuffer_type = renderbuffer_type;
   params->width = width;
   params->height = height;
 
@@ -402,7 +386,7 @@ static gboolean
  * @alloc_params: (allow-none): the #GstAllocationParams for sysmem mappings of the texture
  * @width: the width of the renderbuffer
  * @height: the height of the renderbuffer
- * @renderbuffer_format: the #GstGLFormat for the created textures
+ * @renderbuffer_type: the #GstVideoGLTextureType for the created textures
  *
  * Returns: a new #GstGLRenderbufferAllocationParams for allocating #GstGLRenderbuffer's
  *
@@ -410,7 +394,7 @@ static gboolean
  */
 GstGLRenderbufferAllocationParams *
 gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
-    GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
+    GstAllocationParams * alloc_params, GstVideoGLTextureType renderbuffer_type,
     guint width, guint height)
 {
   GstGLRenderbufferAllocationParams *params =
@@ -422,7 +406,7 @@ gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
           GST_GL_ALLOCATION_PARAMS_ALLOC_FLAG_VIDEO,
           (GstGLAllocationParamsCopyFunc) _gst_gl_rb_alloc_params_copy_data,
           (GstGLAllocationParamsFreeFunc) _gst_gl_rb_alloc_params_free_data,
-          context, alloc_params, width, height, renderbuffer_format, NULL, 0,
+          context, alloc_params, width, height, renderbuffer_type, NULL, 0,
           NULL, NULL)) {
     g_free (params);
     return NULL;
@@ -437,7 +421,7 @@ gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
  * @alloc_params: (allow-none): the #GstAllocationParams for @tex_id
  * @width: the width of the renderbuffer
  * @height: the height of the renderbuffer
- * @renderbuffer_format: the #GstGLFormat for @tex_id
+ * @renderbuffer_type: the #GstVideoGLTextureType for @tex_id
  * @gl_handle: the GL handle to wrap
  * @user_data: (allow-none): user data to call @notify with
  * @notify: (allow-none): a #GDestroyNotify
@@ -449,7 +433,7 @@ gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
  */
 GstGLRenderbufferAllocationParams *
 gst_gl_renderbuffer_allocation_params_new_wrapped (GstGLContext * context,
-    GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
+    GstAllocationParams * alloc_params, GstVideoGLTextureType renderbuffer_type,
     guint width, guint height, gpointer gl_handle, gpointer user_data,
     GDestroyNotify notify)
 {
@@ -462,7 +446,7 @@ gst_gl_renderbuffer_allocation_params_new_wrapped (GstGLContext * context,
           GST_GL_ALLOCATION_PARAMS_ALLOC_FLAG_VIDEO,
           (GstGLAllocationParamsCopyFunc) _gst_gl_rb_alloc_params_copy_data,
           (GstGLAllocationParamsFreeFunc) _gst_gl_rb_alloc_params_free_data,
-          context, alloc_params, width, height, renderbuffer_format, NULL,
+          context, alloc_params, width, height, renderbuffer_type, NULL,
           gl_handle, user_data, notify)) {
     g_free (params);
     return NULL;
