@@ -101,7 +101,10 @@ struct _GstGLWindowPrivate
   GMainLoop *navigation_loop;
   GMutex nav_lock;
   GCond nav_create_cond;
+  GCond nav_destroy_cond;
   gboolean nav_alive;
+  GMutex sync_message_lock;
+  GCond sync_message_cond;
 };
 
 static void gst_gl_window_finalize (GObject * object);
@@ -197,6 +200,9 @@ gst_gl_window_init (GstGLWindow * window)
 
   g_weak_ref_init (&window->context_ref, NULL);
 
+  g_mutex_init (&window->priv->sync_message_lock);
+  g_cond_init (&window->priv->sync_message_cond);
+
   priv->main_context = g_main_context_new ();
   priv->loop = g_main_loop_new (priv->main_context, FALSE);
   priv->navigation_loop = NULL;
@@ -330,12 +336,16 @@ gst_gl_window_finalize (GObject * object)
   GstGLWindowPrivate *priv = window->priv;
 
   GST_INFO ("quit navigation loop");
+
+  g_mutex_lock (&window->priv->nav_lock);
   if (window->priv->navigation_loop) {
     g_main_loop_quit (window->priv->navigation_loop);
     /* wait until navigation thread finished */
-    g_thread_join (window->priv->navigation_thread);
+    while (window->priv->nav_alive)
+      g_cond_wait (&window->priv->nav_destroy_cond, &window->priv->nav_lock);
     window->priv->navigation_thread = NULL;
   }
+  g_mutex_unlock (&window->priv->nav_lock);
 
   if (priv->loop)
     g_main_loop_unref (priv->loop);
@@ -348,6 +358,9 @@ gst_gl_window_finalize (GObject * object)
   g_mutex_clear (&window->lock);
   g_mutex_clear (&window->priv->nav_lock);
   g_cond_clear (&window->priv->nav_create_cond);
+  g_cond_clear (&window->priv->nav_destroy_cond);
+  g_mutex_clear (&window->priv->sync_message_lock);
+  g_cond_clear (&window->priv->sync_message_cond);
   gst_object_unref (window->display);
 
   G_OBJECT_CLASS (gst_gl_window_parent_class)->finalize (object);
@@ -480,8 +493,6 @@ gst_gl_window_draw (GstGLWindow * window)
   }
 
   window_class->draw (window);
-
-  window->queue_resize = FALSE;
 }
 
 /**
@@ -590,8 +601,7 @@ gst_gl_window_quit (GstGLWindow * window)
 
 typedef struct _GstGLSyncMessage
 {
-  GMutex lock;
-  GCond cond;
+  GstGLWindow *window;
   gboolean fired;
 
   GstGLWindowCB callback;
@@ -601,14 +611,14 @@ typedef struct _GstGLSyncMessage
 static void
 _run_message_sync (GstGLSyncMessage * message)
 {
-  g_mutex_lock (&message->lock);
 
   if (message->callback)
     message->callback (message->data);
 
+  g_mutex_lock (&message->window->priv->sync_message_lock);
   message->fired = TRUE;
-  g_cond_signal (&message->cond);
-  g_mutex_unlock (&message->lock);
+  g_cond_broadcast (&message->window->priv->sync_message_cond);
+  g_mutex_unlock (&message->window->priv->sync_message_lock);
 }
 
 void
@@ -617,24 +627,21 @@ gst_gl_window_default_send_message (GstGLWindow * window,
 {
   GstGLSyncMessage message;
 
+  message.window = window;
   message.callback = callback;
   message.data = data;
   message.fired = FALSE;
-  g_mutex_init (&message.lock);
-  g_cond_init (&message.cond);
 
   gst_gl_window_send_message_async (window, (GstGLWindowCB) _run_message_sync,
       &message, NULL);
 
-  g_mutex_lock (&message.lock);
+  g_mutex_lock (&window->priv->sync_message_lock);
 
   /* block until opengl calls have been executed in the gl thread */
   while (!message.fired)
-    g_cond_wait (&message.cond, &message.lock);
-  g_mutex_unlock (&message.lock);
-
-  g_mutex_clear (&message.lock);
-  g_cond_clear (&message.cond);
+    g_cond_wait (&window->priv->sync_message_cond,
+        &window->priv->sync_message_lock);
+  g_mutex_unlock (&window->priv->sync_message_lock);
 }
 
 /**
@@ -938,6 +945,7 @@ gst_gl_window_navigation_thread (GstGLWindow * window)
 
   g_main_loop_run (window->priv->navigation_loop);
 
+  g_mutex_lock (&window->priv->nav_lock);
   g_main_context_pop_thread_default (window->priv->navigation_context);
 
   g_main_loop_unref (window->priv->navigation_loop);
@@ -945,6 +953,10 @@ gst_gl_window_navigation_thread (GstGLWindow * window)
   window->priv->navigation_loop = NULL;
   window->priv->navigation_context = NULL;
 
+  window->priv->nav_alive = FALSE;
+  g_cond_signal (&window->priv->nav_destroy_cond);
+  g_mutex_unlock (&window->priv->nav_lock);
+
   GST_INFO ("navigation loop exited\n");
 
   return NULL;
@@ -1161,14 +1173,38 @@ gst_gl_window_queue_resize (GstGLWindow * window)
     window_class->queue_resize (window);
 }
 
+struct resize_data
+{
+  GstGLWindow *window;
+  guint width, height;
+};
+
+static void
+_on_resize (gpointer data)
+{
+  struct resize_data *resize = data;
+
+  resize->window->resize (resize->window->resize_data, resize->width,
+      resize->height);
+}
+
 void
 gst_gl_window_resize (GstGLWindow * window, guint width, guint height)
 {
   g_return_if_fail (GST_IS_GL_WINDOW (window));
 
-  if (window->resize)
-    window->resize (window->resize_data, width, height);
+  if (window->resize) {
+    struct resize_data resize = { 0, };
+
+    resize.window = window;
+    resize.width = width;
+    resize.height = height;
+
+    gst_gl_window_send_message (window, (GstGLWindowCB) _on_resize, &resize);
+  }
 
   window->priv->surface_width = width;
   window->priv->surface_height = height;
+
+  window->queue_resize = FALSE;
 }
