@@ -25,7 +25,6 @@
 #include <gst/gst.h>
 #include <gst/base/gstbasetransform.h>
 #include <gst/video/video.h>
-#include "gstbayerelements.h"
 #include "gstrgb2bayer.h"
 
 #define GST_CAT_DEFAULT gst_rgb2bayer_debug
@@ -77,8 +76,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_rgb2bayer_parent_class parent_class
 G_DEFINE_TYPE (GstRGB2Bayer, gst_rgb2bayer, GST_TYPE_BASE_TRANSFORM);
-GST_ELEMENT_REGISTER_DEFINE (rgb2bayer, "rgb2bayer", GST_RANK_NONE,
-    gst_rgb2bayer_get_type ());
 
 static void
 gst_rgb2bayer_class_init (GstRGB2BayerClass * klass)
@@ -176,7 +173,7 @@ gst_rgb2bayer_get_unit_size (GstBaseTransform * trans, GstCaps * caps,
     name = gst_structure_get_name (structure);
     /* Our name must be either video/x-bayer video/x-raw */
     if (g_str_equal (name, "video/x-bayer")) {
-      *size = GST_ROUND_UP_4 (width) * height;
+      *size = width * height;
       return TRUE;
     } else {
       /* For output, calculate according to format */
@@ -252,7 +249,7 @@ gst_rgb2bayer_transform (GstBaseTransform * trans, GstBuffer * inbuf,
   src = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
 
   for (j = 0; j < height; j++) {
-    guint8 *dest_line = dest + GST_ROUND_UP_4 (width) * j;
+    guint8 *dest_line = dest + width * j;
     guint8 *src_line = src + frame.info.stride[0] * j;
 
     for (i = 0; i < width; i++) {
