@@ -18,7 +18,7 @@
  */
 
 /*
-gst-launch-0.10 filesrc location=movie.mve ! mvedemux name=d !
+gst-launch-1.0 filesrc location=movie.mve ! mvedemux name=d !
     video/x-raw-rgb ! mvemux quick=true name=m !
     filesink location=test.mve d. ! audio/x-raw-int ! m.
 */
@@ -1350,12 +1350,11 @@ gst_mve_mux_base_init (GstMveMuxClass * klass)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&audio_sink_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &audio_sink_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &video_sink_factory);
 
   gst_element_class_set_static_metadata (element_class, "MVE Multiplexer",
       "Codec/Muxer",
