@@ -430,43 +430,6 @@ done:
   return ret;
 }
 
-static GstBuffer *
-gst_raw_base_parse_align_buffer (GstRawBaseParse * raw_base_parse,
-    gsize alignment, GstBuffer * buffer, gsize out_size)
-{
-  GstMapInfo map;
-
-  gst_buffer_map (buffer, &map, GST_MAP_READ);
-
-  if (map.size < sizeof (guintptr)) {
-    gst_buffer_unmap (buffer, &map);
-    return NULL;
-  }
-
-  if (((guintptr) map.data) & (alignment - 1)) {
-    GstBuffer *new_buffer;
-    GstAllocationParams params = { 0, alignment - 1, 0, 0, };
-
-    new_buffer = gst_buffer_new_allocate (NULL, out_size, &params);
-
-    /* Copy data "by hand", so ensure alignment is kept: */
-    gst_buffer_fill (new_buffer, 0, map.data, out_size);
-
-    gst_buffer_copy_into (new_buffer, buffer, GST_BUFFER_COPY_METADATA, 0,
-        out_size);
-    GST_DEBUG_OBJECT (raw_base_parse,
-        "We want output aligned on %" G_GSIZE_FORMAT ", reallocated",
-        alignment);
-
-    gst_buffer_unmap (buffer, &map);
-
-    return new_buffer;
-  }
-
-  gst_buffer_unmap (buffer, &map);
-
-  return NULL;
-}
 
 static GstFlowReturn
 gst_raw_base_parse_handle_frame (GstBaseParse * parse,
@@ -479,7 +442,6 @@ gst_raw_base_parse_handle_frame (GstBaseParse * parse,
   guint64 buffer_duration;
   GstFlowReturn flow_ret = GST_FLOW_OK;
   GstEvent *new_caps_event = NULL;
-  gint alignment;
   GstRawBaseParse *raw_base_parse = GST_RAW_BASE_PARSE (parse);
   GstRawBaseParseClass *klass = GST_RAW_BASE_PARSE_GET_CLASS (parse);
 
@@ -605,23 +567,6 @@ gst_raw_base_parse_handle_frame (GstBaseParse * parse,
     frame->out_buffer = NULL;
   }
 
-  if (klass->get_alignment
-      && (alignment =
-          klass->get_alignment (raw_base_parse,
-              GST_RAW_BASE_PARSE_CONFIG_CURRENT)) != 1) {
-    GstBuffer *aligned_buffer;
-
-    aligned_buffer =
-        gst_raw_base_parse_align_buffer (raw_base_parse, alignment,
-        frame->out_buffer ? frame->out_buffer : frame->buffer, out_size);
-
-    if (aligned_buffer) {
-      if (frame->out_buffer)
-        gst_buffer_unref (frame->out_buffer);
-      frame->out_buffer = aligned_buffer;
-    }
-  }
-
   /* Set the duration of the output buffer, or if none exists, of
    * the input buffer. Do this after the process() call, since in
    * case out_buffer is set, the subclass has created a new buffer.
