@@ -182,12 +182,11 @@ static GstRawVideoParseConfig
     * gst_raw_video_parse_get_config_ptr (GstRawVideoParse * raw_video_parse,
     GstRawBaseParseConfig config);
 
-static gint gst_raw_video_parse_get_alignment (GstRawBaseParse * raw_base_parse,
-    GstRawBaseParseConfig config);
-
 static void gst_raw_video_parse_init_config (GstRawVideoParseConfig * config);
 static void gst_raw_video_parse_update_info (GstRawVideoParseConfig * config);
 
+
+
 static void
 gst_raw_video_parse_class_init (GstRawVideoParseClass * klass)
 {
@@ -237,8 +236,6 @@ gst_raw_video_parse_class_init (GstRawVideoParseClass * klass)
       GST_DEBUG_FUNCPTR (gst_raw_video_parse_get_units_per_second);
   rawbaseparse_class->get_overhead_size =
       GST_DEBUG_FUNCPTR (gst_raw_video_parse_get_overhead_size);
-  rawbaseparse_class->get_alignment =
-      GST_DEBUG_FUNCPTR (gst_raw_video_parse_get_alignment);
 
   g_object_class_install_property (object_class,
       PROP_WIDTH,
@@ -300,7 +297,7 @@ gst_raw_video_parse_class_init (GstRawVideoParseClass * klass)
       PROP_PLANE_STRIDES,
       g_param_spec_value_array ("plane-strides",
           "Plane strides",
-          "Strides of the planes in bytes",
+          "Strides of the planets in bytes",
           g_param_spec_uint ("plane-stride",
               "Plane stride",
               "Stride of the n-th plane in bytes (0 = stride equals width*bytes-per-pixel)",
@@ -313,7 +310,7 @@ gst_raw_video_parse_class_init (GstRawVideoParseClass * klass)
       PROP_PLANE_OFFSETS,
       g_param_spec_value_array ("plane-offsets",
           "Plane offsets",
-          "Offsets of the planes in bytes",
+          "Offsets of the planets in bytes",
           g_param_spec_uint ("plane-offset",
               "Plane offset",
               "Offset of the n-th plane in bytes",
@@ -932,12 +929,6 @@ gst_raw_video_parse_is_config_ready (GstRawBaseParse * raw_base_parse,
   return gst_raw_video_parse_get_config_ptr (raw_video_parse, config)->ready;
 }
 
-static gint
-gst_raw_video_parse_get_alignment (GstRawBaseParse * raw_base_parse,
-    GstRawBaseParseConfig config)
-{
-  return 32;
-}
 
 static gboolean
 gst_raw_video_parse_process (GstRawBaseParse * raw_base_parse,
