@@ -31,11 +31,9 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch filesrc location=test.ogg ! decodebin ! audioconvert ! speed speed=1.5 ! audioconvert ! audioresample ! autoaudiosink
+ * gst-launch-1.0 filesrc location=test.ogg ! decodebin ! audioconvert ! speed speed=1.5 ! audioconvert ! audioresample ! autoaudiosink
  * ]| Plays an .ogg file at 1.5x speed.
  * </refsect2>
- *
- * Last reviewed on 2007-02-26 (0.10.4.1)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,8 +52,8 @@ GST_DEBUG_CATEGORY_STATIC (speed_debug);
 
 enum
 {
-  ARG_0,
-  ARG_SPEED
+  PROP_0,
+  PROP_SPEED
 };
 
 /* assumption here: sizeof (gfloat) = 4 */
@@ -277,8 +275,12 @@ speed_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_set_position (query, GST_FORMAT_TIME, -1);
 
       if (!gst_pad_peer_query_position (filter->sinkpad, rformat, &cur)) {
-        GST_LOG_OBJECT (filter, "query on peer pad failed");
-        goto error;
+        GST_LOG_OBJECT (filter, "TIME query on peer pad failed, trying BYTES");
+        rformat = GST_FORMAT_BYTES;
+        if (!gst_pad_peer_query_position (filter->sinkpad, rformat, &cur)) {
+          GST_LOG_OBJECT (filter, "BYTES query on peer pad failed too");
+          goto error;
+        }
       }
 
       if (rformat == GST_FORMAT_BYTES)
@@ -324,8 +326,12 @@ speed_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_set_duration (query, GST_FORMAT_TIME, -1);
 
       if (!gst_pad_peer_query_duration (filter->sinkpad, rformat, &end)) {
-        GST_LOG_OBJECT (filter, "query on peer pad failed");
-        goto error;
+        GST_LOG_OBJECT (filter, "TIME query on peer pad failed, trying BYTES");
+        rformat = GST_FORMAT_BYTES;
+        if (!gst_pad_peer_query_duration (filter->sinkpad, rformat, &end)) {
+          GST_LOG_OBJECT (filter, "BYTES query on peer pad failed too");
+          goto error;
+        }
       }
 
       if (rformat == GST_FORMAT_BYTES)
@@ -382,9 +388,9 @@ gst_speed_class_init (GstSpeedClass * klass)
   gobject_class->get_property = speed_get_property;
   gstelement_class->change_state = speed_change_state;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SPEED,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SPEED,
       g_param_spec_float ("speed", "speed", "speed",
-          0.1, 40.0, 1.0,
+          0.1f, 40.0, 1.0,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_set_static_metadata (gstelement_class, "Speed",
@@ -393,10 +399,10 @@ gst_speed_class_init (GstSpeedClass * klass)
       "Andy Wingo <apwingo@eos.ncsu.edu>, "
       "Tim-Philipp Müller <tim@centricular.net>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_speed_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_speed_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_speed_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_speed_sink_template);
 }
 
 static void
@@ -564,7 +570,7 @@ speed_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
         return ret;
       }
     }
-
+      /* Fallthrough so that the caps event gets forwarded */
     default:
       ret = gst_pad_event_default (pad, parent, event);
       break;
@@ -635,7 +641,7 @@ speed_set_property (GObject * object, guint prop_id, const GValue * value,
   GstSpeed *filter = GST_SPEED (object);
 
   switch (prop_id) {
-    case ARG_SPEED:
+    case PROP_SPEED:
       filter->speed = g_value_get_float (value);
       break;
     default:
@@ -652,7 +658,7 @@ speed_get_property (GObject * object, guint prop_id, GValue * value,
   GstSpeed *filter = GST_SPEED (object);
 
   switch (prop_id) {
-    case ARG_SPEED:
+    case PROP_SPEED:
       g_value_set_float (value, filter->speed);
       break;
     default:
