@@ -29,7 +29,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v -m avfassetsrc uri="file://movie.mp4" ! autovideosink
+ * gst-launch-1.0 -v -m avfassetsrc uri="file://movie.mp4" ! autovideosink
  * ]|
  * </refsect2>
  */
@@ -149,10 +149,8 @@ gst_avf_asset_src_class_init (GstAVFAssetSrcClass * klass)
     "Read and decode samples from AVFoundation assets using the AVFAssetReader API",
     "Andoni Morales Alastruey amorales@fluendo.com");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&audio_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&video_factory));
+  gst_element_class_add_static_pad_template (gstelement_class, &audio_factory);
+  gst_element_class_add_static_pad_template (gstelement_class, &video_factory);
 
   gobject_class->set_property = gst_avf_asset_src_set_property;
   gobject_class->get_property = gst_avf_asset_src_get_property;
@@ -208,9 +206,7 @@ gst_avf_asset_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_URI:
-      if (self->uri) {
-        g_free (self->uri);
-      }
+      g_free (self->uri);
       self->uri = g_value_dup_string (value);
       break;
     default:
@@ -539,8 +535,7 @@ gst_avf_asset_src_read_data (GstAVFAssetSrc *self, GstPad *pad,
     }
 
     if (combined_ret != GST_FLOW_OK) {
-      GST_ELEMENT_ERROR (self, STREAM, FAILED, ("Internal data stream error."),
-          ("stream stopped reason %s", gst_flow_get_name (ret)));
+      GST_ELEMENT_FLOW_ERROR (self, ret);
     }
 
     gst_pad_pause_task (pad);
@@ -808,25 +803,20 @@ gst_avf_asset_src_uri_set_uri (GstURIHandler * handler, const gchar * uri, GErro
   NSString *str;
   NSURL *url;
   AVAsset *asset;
-  gchar *escaped_uri;
   gboolean ret = FALSE;
 
   OBJC_CALLOUT_BEGIN ();
-  escaped_uri = g_uri_escape_string (uri, ":/", TRUE);
-  str = [NSString stringWithUTF8String: escaped_uri];
+  str = [NSString stringWithUTF8String: uri];
   url = [[NSURL alloc] initWithString: str];
   asset = [AVAsset assetWithURL: url];
-  g_free (escaped_uri);
 
   if (asset.playable) {
     ret = TRUE;
-    if (self->uri) {
-      g_free (self->uri);
-    }
+    g_free (self->uri);
     self->uri = g_strdup (uri);
   } else {
     g_set_error (error, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
-        "Invalid URI '%s' for avfassetsrc", self->uri);
+        "Invalid URI '%s' for avfassetsrc", uri);
   }
   OBJC_CALLOUT_END ();
   return ret;
@@ -909,16 +899,13 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 {
   NSString *str;
   NSURL *url;
-  gchar *escaped_uri;
 
-  GST_INFO ("Initializing AVFAssetReader with uri:%s", uri);
+  GST_INFO ("Initializing AVFAssetReader with uri: %s", uri);
   *error = NULL;
 
-  escaped_uri = g_uri_escape_string (uri, ":/", TRUE);
-  str = [NSString stringWithUTF8String: escaped_uri];
+  str = [NSString stringWithUTF8String: uri];
   url = [[NSURL alloc] initWithString: str];
   asset = [[AVAsset assetWithURL: url] retain];
-  g_free (escaped_uri);
 
   if (!asset.playable) {
     *error = g_error_new (GST_AVF_ASSET_SRC_ERROR, GST_AVF_ASSET_ERROR_NOT_PLAYABLE,
@@ -1083,7 +1070,11 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
     return NULL;
   }
 
-  buf = gst_core_media_buffer_new (cmbuf, FALSE, TRUE);
+  buf = gst_core_media_buffer_new (cmbuf, FALSE, NULL);
+  CFRelease (cmbuf);
+  if (buf == NULL)
+    return NULL;
+  /* cmbuf is now retained by buf (in meta) */
   dur = CMSampleBufferGetDuration (cmbuf);
   ts = CMSampleBufferGetPresentationTimeStamp (cmbuf);
   if (dur.value != 0) {
