@@ -17,23 +17,6 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
  * Boston, MA 02110-1335, USA.
  */
-/**
- * SECTION:element-decklinkaudiosrc
- * @short_description: Inputs Audio from a BlackMagic DeckLink Device
- * @see_also: decklinkvideosrc
- *
- * Capture Video and Audio from a BlackMagic DeckLink Device. Can only be used
- * in conjunction with decklinkvideosink.
- *
- * ## Sample pipeline
- * |[
- * gst-launch-1.0 \
- *   decklinkvideosrc device-number=0 mode=1080p25 ! autovideosink \
- *   decklinkaudiosrc device-number=0 ! autoaudiosink
- * ]|
- * Capturing 1080p25 video and audio from the SDI-In of Card 0. Devices are numbered
- * starting with 0.
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -51,11 +34,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_decklink_audio_src_debug);
 
 #define DEFAULT_ALIGNMENT_THRESHOLD   (40 * GST_MSECOND)
 #define DEFAULT_DISCONT_WAIT          (1 * GST_SECOND)
-#define DEFAULT_CHANNELS              (GST_DECKLINK_AUDIO_CHANNELS_2)
-
-#ifndef ABSDIFF
-#define ABSDIFF(x, y) ( (x) > (y) ? ((x) - (y)) : ((y) - (x)) )
-#endif
 
 enum
 {
@@ -64,9 +42,7 @@ enum
   PROP_DEVICE_NUMBER,
   PROP_ALIGNMENT_THRESHOLD,
   PROP_DISCONT_WAIT,
-  PROP_BUFFER_SIZE,
-  PROP_CHANNELS,
-  PROP_HW_SERIAL_NUMBER
+  PROP_BUFFER_SIZE
 };
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("src",
@@ -74,27 +50,23 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS
     ("audio/x-raw, format={S16LE,S32LE}, channels=2, rate=48000, "
-        "layout=interleaved;"
-        "audio/x-raw, format={S16LE,S32LE}, channels={8,16}, channel-mask=(bitmask)0, rate=48000, "
         "layout=interleaved")
     );
 
 typedef struct
 {
   IDeckLinkAudioInputPacket *packet;
-  GstClockTime timestamp;
-  GstClockTime stream_timestamp;
-  GstClockTime stream_duration;
-  GstClockTime hardware_timestamp;
-  GstClockTime hardware_duration;
-  gboolean no_signal;
+  GstClockTime capture_time;
+  gboolean discont;
 } CapturePacket;
 
 static void
-capture_packet_clear (CapturePacket * packet)
+capture_packet_free (void *data)
 {
+  CapturePacket *packet = (CapturePacket *) data;
+
   packet->packet->Release ();
-  memset (packet, 0, sizeof (*packet));
+  g_free (packet);
 }
 
 typedef struct
@@ -123,6 +95,10 @@ static GstStateChangeReturn
 gst_decklink_audio_src_change_state (GstElement * element,
     GstStateChange transition);
 
+static gboolean gst_decklink_audio_src_set_caps (GstBaseSrc * bsrc,
+    GstCaps * caps);
+static GstCaps *gst_decklink_audio_src_get_caps (GstBaseSrc * bsrc,
+    GstCaps * filter);
 static gboolean gst_decklink_audio_src_unlock (GstBaseSrc * bsrc);
 static gboolean gst_decklink_audio_src_unlock_stop (GstBaseSrc * bsrc);
 static gboolean gst_decklink_audio_src_query (GstBaseSrc * bsrc,
@@ -154,8 +130,9 @@ gst_decklink_audio_src_class_init (GstDecklinkAudioSrcClass * klass)
   element_class->change_state =
       GST_DEBUG_FUNCPTR (gst_decklink_audio_src_change_state);
 
+  basesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_decklink_audio_src_get_caps);
+  basesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_decklink_audio_src_set_caps);
   basesrc_class->query = GST_DEBUG_FUNCPTR (gst_decklink_audio_src_query);
-  basesrc_class->negotiate = NULL;
   basesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_decklink_audio_src_unlock);
   basesrc_class->unlock_stop =
       GST_DEBUG_FUNCPTR (gst_decklink_audio_src_unlock_stop);
@@ -194,23 +171,10 @@ gst_decklink_audio_src_class_init (GstDecklinkAudioSrcClass * klass)
           G_MAXINT, DEFAULT_BUFFER_SIZE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
-  g_object_class_install_property (gobject_class, PROP_CHANNELS,
-      g_param_spec_enum ("channels", "Channels",
-          "Audio channels",
-          GST_TYPE_DECKLINK_AUDIO_CHANNELS, DEFAULT_CHANNELS,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
-              G_PARAM_CONSTRUCT)));
-
-  g_object_class_install_property (gobject_class, PROP_HW_SERIAL_NUMBER,
-      g_param_spec_string ("hw-serial-number", "Hardware serial number",
-          "The serial number (hardware ID) of the Decklink card",
-          NULL, (GParamFlags) (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
-
   gst_element_class_add_static_pad_template (element_class, &sink_template);
 
   gst_element_class_set_static_metadata (element_class, "Decklink Audio Source",
-      "Audio/Source/Hardware", "Decklink Source",
-      "David Schleef <ds@entropywave.com>, "
+      "Audio/Src", "Decklink Source", "David Schleef <ds@entropywave.com>, "
       "Sebastian Dröge <sebastian@centricular.com>");
 
   GST_DEBUG_CATEGORY_INIT (gst_decklink_audio_src_debug, "decklinkaudiosrc",
@@ -224,19 +188,14 @@ gst_decklink_audio_src_init (GstDecklinkAudioSrc * self)
   self->alignment_threshold = DEFAULT_ALIGNMENT_THRESHOLD;
   self->discont_wait = DEFAULT_DISCONT_WAIT;
   self->buffer_size = DEFAULT_BUFFER_SIZE;
-  self->channels = DEFAULT_CHANNELS;
 
   gst_base_src_set_live (GST_BASE_SRC (self), TRUE);
   gst_base_src_set_format (GST_BASE_SRC (self), GST_FORMAT_TIME);
 
-  gst_pad_use_fixed_caps (GST_BASE_SRC_PAD (self));
-
   g_mutex_init (&self->lock);
   g_cond_init (&self->cond);
 
-  self->current_packets =
-      gst_queue_array_new_for_struct (sizeof (CapturePacket),
-      DEFAULT_BUFFER_SIZE);
+  g_queue_init (&self->current_packets);
 }
 
 void
@@ -262,9 +221,6 @@ gst_decklink_audio_src_set_property (GObject * object, guint property_id,
     case PROP_BUFFER_SIZE:
       self->buffer_size = g_value_get_uint (value);
       break;
-    case PROP_CHANNELS:
-      self->channels = (GstDecklinkAudioChannelsEnum) g_value_get_enum (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -293,15 +249,6 @@ gst_decklink_audio_src_get_property (GObject * object, guint property_id,
     case PROP_BUFFER_SIZE:
       g_value_set_uint (value, self->buffer_size);
       break;
-    case PROP_CHANNELS:
-      g_value_set_enum (value, self->channels);
-      break;
-    case PROP_HW_SERIAL_NUMBER:
-      if (self->input)
-        g_value_set_string (value, self->input->hw_serial_number);
-      else
-        g_value_set_string (value, NULL);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -315,56 +262,42 @@ gst_decklink_audio_src_finalize (GObject * object)
 
   g_mutex_clear (&self->lock);
   g_cond_clear (&self->cond);
-  if (self->current_packets) {
-    while (gst_queue_array_get_length (self->current_packets) > 0) {
-      CapturePacket *tmp = (CapturePacket *)
-          gst_queue_array_pop_head_struct (self->current_packets);
-      capture_packet_clear (tmp);
-    }
-    gst_queue_array_free (self->current_packets);
-    self->current_packets = NULL;
-  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
-gst_decklink_audio_src_start (GstDecklinkAudioSrc * self)
+gst_decklink_audio_src_set_caps (GstBaseSrc * bsrc, GstCaps * caps)
 {
+  GstDecklinkAudioSrc *self = GST_DECKLINK_AUDIO_SRC_CAST (bsrc);
   BMDAudioSampleType sample_depth;
+  GstCaps *current_caps;
   HRESULT ret;
   BMDAudioConnection conn = (BMDAudioConnection) - 1;
-  GstCaps *allowed_caps, *caps;
 
-  g_mutex_lock (&self->input->lock);
-  if (self->input->audio_enabled) {
-    g_mutex_unlock (&self->input->lock);
-    return TRUE;
-  }
-  g_mutex_unlock (&self->input->lock);
+  GST_DEBUG_OBJECT (self, "Setting caps %" GST_PTR_FORMAT, caps);
 
-  /* Negotiate the format / sample depth with downstream */
-  allowed_caps = gst_pad_get_allowed_caps (GST_BASE_SRC_PAD (self));
-  if (!allowed_caps)
-    allowed_caps = gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD (self));
+  if ((current_caps = gst_pad_get_current_caps (GST_BASE_SRC_PAD (bsrc)))) {
+    GST_DEBUG_OBJECT (self, "Pad already has caps %" GST_PTR_FORMAT, caps);
 
-  sample_depth = bmdAudioSampleType32bitInteger;
-  if (!gst_caps_is_empty (allowed_caps)) {
-    GstStructure *s;
-
-    allowed_caps = gst_caps_simplify (allowed_caps);
-
-    s = gst_caps_get_structure (allowed_caps, 0);
-
-    /* If it's not a string then both formats are supported */
-    if (gst_structure_has_field_typed (s, "format", G_TYPE_STRING)) {
-      const gchar *format = gst_structure_get_string (s, "format");
-      if (g_str_equal (format, "S16LE")) {
-        sample_depth = bmdAudioSampleType16bitInteger;
-      }
+    if (!gst_caps_is_equal (caps, current_caps)) {
+      GST_ERROR_OBJECT (self, "New caps are not equal to old caps");
+      gst_caps_unref (current_caps);
+      return FALSE;
+    } else {
+      gst_caps_unref (current_caps);
+      return TRUE;
     }
   }
-  gst_caps_unref (allowed_caps);
+
+  if (!gst_audio_info_from_caps (&self->info, caps))
+    return FALSE;
+
+  if (self->info.finfo->format == GST_AUDIO_FORMAT_S16LE) {
+    sample_depth = bmdAudioSampleType16bitInteger;
+  } else {
+    sample_depth = bmdAudioSampleType32bitInteger;
+  }
 
   switch (self->connection) {
     case GST_DECKLINK_AUDIO_CONNECTION_AUTO:{
@@ -434,23 +367,17 @@ gst_decklink_audio_src_start (GstDecklinkAudioSrc * self)
         self->input->config->SetInt (bmdDeckLinkConfigAudioInputConnection,
         conn);
     if (ret != S_OK) {
-      GST_ERROR ("set configuration (audio input connection): 0x%08lx",
-          (unsigned long) ret);
+      GST_ERROR ("set configuration (audio input connection): 0x%08x", ret);
       return FALSE;
     }
   }
 
   ret = self->input->input->EnableAudioInput (bmdAudioSampleRate48kHz,
-      sample_depth, self->channels_found);
+      sample_depth, 2);
   if (ret != S_OK) {
-    GST_WARNING_OBJECT (self, "Failed to enable audio input: 0x%08lx",
-        (unsigned long) ret);
+    GST_WARNING_OBJECT (self, "Failed to enable audio input: 0x%08x", ret);
     return FALSE;
   }
-  gst_audio_info_set_format (&self->info,
-      sample_depth ==
-      bmdAudioSampleType16bitInteger ? GST_AUDIO_FORMAT_S16LE :
-      GST_AUDIO_FORMAT_S32LE, 48000, self->channels_found, NULL);
 
   g_mutex_lock (&self->input->lock);
   self->input->audio_enabled = TRUE;
@@ -458,104 +385,72 @@ gst_decklink_audio_src_start (GstDecklinkAudioSrc * self)
     self->input->start_streams (self->input->videosrc);
   g_mutex_unlock (&self->input->lock);
 
-  caps = gst_audio_info_to_caps (&self->info);
-  if (!gst_base_src_set_caps (GST_BASE_SRC (self), caps)) {
-    gst_caps_unref (caps);
-    GST_WARNING_OBJECT (self, "Failed to set caps");
-    return FALSE;
-  }
-  gst_caps_unref (caps);
-
   return TRUE;
 }
 
+static GstCaps *
+gst_decklink_audio_src_get_caps (GstBaseSrc * bsrc, GstCaps * filter)
+{
+  GstCaps *caps;
+
+  // We don't support renegotiation
+  caps = gst_pad_get_current_caps (GST_BASE_SRC_PAD (bsrc));
+
+  if (!caps)
+    caps = gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD (bsrc));
+
+  if (filter) {
+    GstCaps *tmp =
+        gst_caps_intersect_full (filter, caps, GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (caps);
+    caps = tmp;
+  }
+
+  return caps;
+}
+
 static void
 gst_decklink_audio_src_got_packet (GstElement * element,
     IDeckLinkAudioInputPacket * packet, GstClockTime capture_time,
-    GstClockTime stream_time, GstClockTime stream_duration,
-    GstClockTime hardware_time, GstClockTime hardware_duration,
-    gboolean no_signal)
+    gboolean discont)
 {
   GstDecklinkAudioSrc *self = GST_DECKLINK_AUDIO_SRC_CAST (element);
-  GstClockTime timestamp;
+  GstDecklinkVideoSrc *videosrc = NULL;
 
-  GST_LOG_OBJECT (self,
-      "Got audio packet at %" GST_TIME_FORMAT " / %" GST_TIME_FORMAT
-      ", no signal %d", GST_TIME_ARGS (capture_time),
-      GST_TIME_ARGS (stream_time), no_signal);
+  GST_LOG_OBJECT (self, "Got audio packet at %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (capture_time));
 
   g_mutex_lock (&self->input->lock);
-  if (self->input->videosrc) {
-    GstDecklinkVideoSrc *videosrc =
+  if (self->input->videosrc)
+    videosrc =
         GST_DECKLINK_VIDEO_SRC_CAST (gst_object_ref (self->input->videosrc));
-
-    if (videosrc->drop_no_signal_frames && no_signal) {
-      g_mutex_unlock (&self->input->lock);
-      return;
-    }
-
-    if (videosrc->first_time == GST_CLOCK_TIME_NONE)
-      videosrc->first_time = stream_time;
-
-    if (videosrc->skip_first_time > 0
-        && stream_time - videosrc->first_time < videosrc->skip_first_time) {
-      GST_DEBUG_OBJECT (self,
-          "Skipping frame as requested: %" GST_TIME_FORMAT " < %"
-          GST_TIME_FORMAT, GST_TIME_ARGS (stream_time),
-          GST_TIME_ARGS (videosrc->skip_first_time + videosrc->first_time));
-      g_mutex_unlock (&self->input->lock);
-      return;
-    }
-
-    if (videosrc->output_stream_time)
-      timestamp = stream_time;
-    else
-      timestamp = gst_clock_adjust_with_calibration (NULL, stream_time,
-          videosrc->current_time_mapping.xbase,
-          videosrc->current_time_mapping.b, videosrc->current_time_mapping.num,
-          videosrc->current_time_mapping.den);
-  } else {
-    timestamp = capture_time;
-  }
   g_mutex_unlock (&self->input->lock);
 
-  GST_LOG_OBJECT (self, "Converted times to %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (timestamp));
+  if (videosrc) {
+    gst_decklink_video_src_convert_to_external_clock (videosrc, &capture_time,
+        NULL);
+    gst_object_unref (videosrc);
+    GST_LOG_OBJECT (self, "Actual timestamp %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (capture_time));
+  }
 
   g_mutex_lock (&self->lock);
   if (!self->flushing) {
-    CapturePacket p;
-    guint skipped_packets = 0;
-    GstClockTime from_timestamp = GST_CLOCK_TIME_NONE;
-    GstClockTime to_timestamp = GST_CLOCK_TIME_NONE;
+    CapturePacket *p;
 
-    while (gst_queue_array_get_length (self->current_packets) >=
-        self->buffer_size) {
-      CapturePacket *tmp = (CapturePacket *)
-          gst_queue_array_pop_head_struct (self->current_packets);
-      if (skipped_packets == 0)
-        from_timestamp = tmp->timestamp;
-      skipped_packets++;
-      to_timestamp = tmp->timestamp;
-      capture_packet_clear (tmp);
+    while (g_queue_get_length (&self->current_packets) >= self->buffer_size) {
+      p = (CapturePacket *) g_queue_pop_head (&self->current_packets);
+      GST_WARNING_OBJECT (self, "Dropping old packet at %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (p->capture_time));
+      capture_packet_free (p);
     }
 
-    if (skipped_packets > 0)
-      GST_WARNING_OBJECT (self,
-          "Dropped %u old packets from %" GST_TIME_FORMAT " to %"
-          GST_TIME_FORMAT, skipped_packets, GST_TIME_ARGS (from_timestamp),
-          GST_TIME_ARGS (to_timestamp));
-
-    memset (&p, 0, sizeof (p));
-    p.packet = packet;
-    p.timestamp = timestamp;
-    p.stream_timestamp = stream_time;
-    p.stream_duration = stream_duration;
-    p.hardware_timestamp = hardware_time;
-    p.hardware_duration = hardware_duration;
-    p.no_signal = no_signal;
+    p = (CapturePacket *) g_malloc0 (sizeof (CapturePacket));
+    p->packet = packet;
+    p->capture_time = capture_time;
+    p->discont = discont;
     packet->AddRef ();
-    gst_queue_array_push_tail_struct (self->current_packets, &p);
+    g_queue_push_tail (&self->current_packets, p);
     g_cond_signal (&self->cond);
   }
   g_mutex_unlock (&self->lock);
@@ -569,46 +464,39 @@ gst_decklink_audio_src_create (GstPushSrc * bsrc, GstBuffer ** buffer)
   const guint8 *data;
   glong sample_count;
   gsize data_size;
-  CapturePacket p;
+  CapturePacket *p;
   AudioPacket *ap;
   GstClockTime timestamp, duration;
   GstClockTime start_time, end_time;
   guint64 start_offset, end_offset;
   gboolean discont = FALSE;
-  static GstStaticCaps stream_reference =
-      GST_STATIC_CAPS ("timestamp/x-decklink-stream");
-  static GstStaticCaps hardware_reference =
-      GST_STATIC_CAPS ("timestamp/x-decklink-hardware");
-
-  if (!gst_decklink_audio_src_start (self)) {
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
 
 retry:
   g_mutex_lock (&self->lock);
-  while (gst_queue_array_is_empty (self->current_packets) && !self->flushing) {
+  while (g_queue_is_empty (&self->current_packets) && !self->flushing) {
     g_cond_wait (&self->cond, &self->lock);
   }
 
+  p = (CapturePacket *) g_queue_pop_head (&self->current_packets);
+  g_mutex_unlock (&self->lock);
+
   if (self->flushing) {
+    if (p)
+      capture_packet_free (p);
     GST_DEBUG_OBJECT (self, "Flushing");
-    g_mutex_unlock (&self->lock);
     return GST_FLOW_FLUSHING;
   }
 
-  p = *(CapturePacket *)
-      gst_queue_array_pop_head_struct (self->current_packets);
-  g_mutex_unlock (&self->lock);
-
-  p.packet->GetBytes ((gpointer *) & data);
-  sample_count = p.packet->GetSampleFrameCount ();
+  p->packet->GetBytes ((gpointer *) & data);
+  sample_count = p->packet->GetSampleFrameCount ();
   data_size = self->info.bpf * sample_count;
 
-  if (p.timestamp == GST_CLOCK_TIME_NONE && self->next_offset == (guint64) - 1) {
+  if (p->capture_time == GST_CLOCK_TIME_NONE
+      && self->next_offset == (guint64) - 1) {
     GST_DEBUG_OBJECT (self,
         "Got packet without timestamp before initial "
         "timestamp after discont - dropping");
-    capture_packet_clear (&p);
+    capture_packet_free (p);
     goto retry;
   }
 
@@ -619,12 +507,13 @@ retry:
       (gpointer) data, data_size, 0, data_size, ap,
       (GDestroyNotify) audio_packet_free);
 
-  ap->packet = p.packet;
-  p.packet->AddRef ();
+  ap->packet = p->packet;
+  p->packet->AddRef ();
   ap->input = self->input->input;
   ap->input->AddRef ();
 
-  timestamp = p.timestamp;
+  timestamp = p->capture_time;
+  discont = p->discont;
 
   // Jitter and discontinuity handling, based on audiobasesrc
   start_time = timestamp;
@@ -632,8 +521,6 @@ retry:
   // Convert to the sample numbers
   start_offset =
       gst_util_uint64_scale (start_time, self->info.rate, GST_SECOND);
-  // Convert back to round down to a sample multiple and get rid of rounding errors
-  start_time = gst_util_uint64_scale (start_offset, GST_SECOND, self->info.rate);
 
   end_offset = start_offset + sample_count;
   end_time = gst_util_uint64_scale_int (end_offset, GST_SECOND,
@@ -641,7 +528,7 @@ retry:
 
   duration = end_time - start_time;
 
-  if (self->next_offset == (guint64) - 1) {
+  if (discont || self->next_offset == (guint64) - 1) {
     discont = TRUE;
   } else {
     guint64 diff, max_sample_diff;
@@ -694,88 +581,15 @@ retry:
         self->info.rate) - timestamp;
   }
 
-  // Detect gaps in stream time
-  self->processed += sample_count;
-  if (self->expected_stream_time != GST_CLOCK_TIME_NONE
-      && p.stream_timestamp == GST_CLOCK_TIME_NONE) {
-    /* We missed a frame. Extrapolate the timestamps */
-    p.stream_timestamp = self->expected_stream_time;
-    p.stream_duration =
-        gst_util_uint64_scale_int (sample_count, GST_SECOND, self->info.rate);
-  }
-  if (self->last_hardware_time != GST_CLOCK_TIME_NONE
-      && p.hardware_timestamp == GST_CLOCK_TIME_NONE) {
-    /* This should always happen when the previous one also does, but let's
-     * have two separate checks just in case */
-    GstClockTime start_hw_offset, end_hw_offset;
-    start_hw_offset =
-        gst_util_uint64_scale (self->last_hardware_time, self->info.rate,
-        GST_SECOND);
-    end_hw_offset = start_hw_offset + sample_count;
-    p.hardware_timestamp =
-        gst_util_uint64_scale_int (end_hw_offset, GST_SECOND, self->info.rate);
-    /* Will be the same as the stream duration - reuse it */
-    p.hardware_duration = p.stream_duration;
-  }
-
-  if (p.stream_timestamp != GST_CLOCK_TIME_NONE) {
-    GstClockTime start_stream_time, end_stream_time;
-
-    start_stream_time = p.stream_timestamp;
-
-    start_offset =
-        gst_util_uint64_scale (start_stream_time, self->info.rate, GST_SECOND);
-
-    end_offset = start_offset + sample_count;
-    end_stream_time = gst_util_uint64_scale_int (end_offset, GST_SECOND,
-        self->info.rate);
-
-    /* With drop-frame we have gaps of 1 sample every now and then (rounding
-     * errors because of the samples-per-frame pattern which is not 100%
-     * accurate), and due to rounding errors in the calculations these can be
-     * 2>x>1 */
-    if (self->expected_stream_time != GST_CLOCK_TIME_NONE &&
-        ABSDIFF (self->expected_stream_time, p.stream_timestamp) >
-        gst_util_uint64_scale (2, GST_SECOND, self->info.rate)) {
-      GstMessage *msg;
-      GstClockTime running_time;
-
-      self->dropped +=
-          gst_util_uint64_scale (ABSDIFF (self->expected_stream_time,
-              p.stream_timestamp), self->info.rate, GST_SECOND);
-      running_time =
-          gst_segment_to_running_time (&GST_BASE_SRC (self)->segment,
-          GST_FORMAT_TIME, timestamp);
-
-      msg =
-          gst_message_new_qos (GST_OBJECT (self), TRUE, running_time,
-          p.stream_timestamp, timestamp, duration);
-      gst_message_set_qos_stats (msg, GST_FORMAT_DEFAULT, self->processed,
-          self->dropped);
-      gst_element_post_message (GST_ELEMENT (self), msg);
-    }
-    self->expected_stream_time = end_stream_time;
-  }
-  self->last_hardware_time = p.hardware_timestamp;
-
-  if (p.no_signal)
-    GST_BUFFER_FLAG_SET (*buffer, GST_BUFFER_FLAG_GAP);
   GST_BUFFER_TIMESTAMP (*buffer) = timestamp;
   GST_BUFFER_DURATION (*buffer) = duration;
 
-  gst_buffer_add_reference_timestamp_meta (*buffer,
-      gst_static_caps_get (&stream_reference), p.stream_timestamp,
-      p.stream_duration);
-  gst_buffer_add_reference_timestamp_meta (*buffer,
-      gst_static_caps_get (&hardware_reference), p.hardware_timestamp,
-      p.hardware_duration);
-
   GST_DEBUG_OBJECT (self,
       "Outputting buffer %p with timestamp %" GST_TIME_FORMAT " and duration %"
       GST_TIME_FORMAT, *buffer, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (*buffer)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (*buffer)));
 
-  capture_packet_clear (&p);
+  capture_packet_free (p);
 
   return flow_ret;
 }
@@ -838,11 +652,8 @@ gst_decklink_audio_src_unlock_stop (GstBaseSrc * bsrc)
 
   g_mutex_lock (&self->lock);
   self->flushing = FALSE;
-  while (gst_queue_array_get_length (self->current_packets) > 0) {
-    CapturePacket *tmp = (CapturePacket *)
-        gst_queue_array_pop_head_struct (self->current_packets);
-    capture_packet_clear (tmp);
-  }
+  g_queue_foreach (&self->current_packets, (GFunc) capture_packet_free, NULL);
+  g_queue_clear (&self->current_packets);
   g_mutex_unlock (&self->lock);
 
   return TRUE;
@@ -861,30 +672,7 @@ gst_decklink_audio_src_open (GstDecklinkAudioSrc * self)
     return FALSE;
   }
 
-  g_object_notify (G_OBJECT (self), "hw-serial-number");
-
   g_mutex_lock (&self->input->lock);
-  if (self->channels > 0) {
-    self->channels_found = self->channels;
-  } else {
-    if (self->input->attributes) {
-      int64_t channels_found;
-
-      HRESULT ret = self->input->attributes->GetInt
-          (BMDDeckLinkMaximumAudioChannels, &channels_found);
-      self->channels_found = channels_found;
-
-      /* Sometimes the card may report an invalid number of channels. In
-       * that case, we should (empirically) use 8. */
-      if (ret != S_OK ||
-          self->channels_found == 0 || g_enum_get_value ((GEnumClass *)
-              g_type_class_peek (GST_TYPE_DECKLINK_AUDIO_CHANNELS),
-              self->channels_found)
-          == NULL) {
-        self->channels_found = GST_DECKLINK_AUDIO_CHANNELS_8;
-      }
-    }
-  }
   self->input->got_audio_packet = gst_decklink_audio_src_got_packet;
   g_mutex_unlock (&self->input->lock);
 
@@ -914,11 +702,8 @@ gst_decklink_audio_src_stop (GstDecklinkAudioSrc * self)
 {
   GST_DEBUG_OBJECT (self, "Stopping");
 
-  while (gst_queue_array_get_length (self->current_packets) > 0) {
-    CapturePacket *tmp = (CapturePacket *)
-        gst_queue_array_pop_head_struct (self->current_packets);
-    capture_packet_clear (tmp);
-  }
+  g_queue_foreach (&self->current_packets, (GFunc) capture_packet_free, NULL);
+  g_queue_clear (&self->current_packets);
 
   if (self->input && self->input->audio_enabled) {
     g_mutex_lock (&self->input->lock);
@@ -964,9 +749,6 @@ gst_decklink_audio_src_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      self->processed = 0;
-      self->dropped = 0;
-      self->expected_stream_time = GST_CLOCK_TIME_NONE;
       if (!gst_decklink_audio_src_open (self)) {
         ret = GST_STATE_CHANGE_FAILURE;
         goto out;
