@@ -495,31 +495,26 @@ test_play_audio_video_eos_cb (GstPlayer * player, TestPlayerStateChange change,
           GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
       break;
     case 4:
-      fail_unless_equals_int (change, STATE_CHANGE_MEDIA_INFO_UPDATED);
-      new_state->test_data =
-          GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
-      break;
-    case 5:
       fail_unless_equals_int (change, STATE_CHANGE_DURATION_CHANGED);
       fail_unless_equals_uint64 (new_state->duration,
           G_GUINT64_CONSTANT (464399092));
       new_state->test_data =
           GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
       break;
-    case 6:
+    case 5:
       fail_unless_equals_int (change, STATE_CHANGE_POSITION_UPDATED);
       fail_unless_equals_uint64 (new_state->position, G_GUINT64_CONSTANT (0));
       new_state->test_data =
           GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
       break;
-    case 7:
+    case 6:
       fail_unless_equals_int (change, STATE_CHANGE_STATE_CHANGED);
       fail_unless_equals_int (old_state->state, GST_PLAYER_STATE_BUFFERING);
       fail_unless_equals_int (new_state->state, GST_PLAYER_STATE_PLAYING);
       new_state->test_data =
           GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
       break;
-    case 8:
+    case 7:
       if (change == STATE_CHANGE_POSITION_UPDATED) {
         fail_unless (old_state->position <= new_state->position);
       } else {
@@ -529,7 +524,7 @@ test_play_audio_video_eos_cb (GstPlayer * player, TestPlayerStateChange change,
             GINT_TO_POINTER ((video ? 0x10 : 0x00) | (step + 1));
       }
       break;
-    case 9:
+    case 8:
       fail_unless_equals_int (change, STATE_CHANGE_STATE_CHANGED);
       fail_unless_equals_int (old_state->state, GST_PLAYER_STATE_PLAYING);
       fail_unless_equals_int (new_state->state, GST_PLAYER_STATE_STOPPED);
@@ -566,7 +561,7 @@ START_TEST (test_play_audio_eos)
   gst_player_play (player);
   g_main_loop_run (state.loop);
 
-  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data), 10);
+  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data), 9);
 
   stop_player (player, &state);
   g_object_unref (player);
@@ -581,7 +576,7 @@ test_audio_info (GstPlayerMediaInfo * media_info)
   gint i = 0;
   GList *list;
 
-  for (list = gst_player_media_info_get_audio_streams (media_info);
+  for (list = gst_player_get_audio_streams (media_info);
       list != NULL; list = list->next) {
     GstPlayerStreamInfo *stream = (GstPlayerStreamInfo *) list->data;
     GstPlayerAudioInfo *audio_info = (GstPlayerAudioInfo *) stream;
@@ -620,7 +615,7 @@ test_video_info (GstPlayerMediaInfo * media_info)
 {
   GList *list;
 
-  for (list = gst_player_media_info_get_video_streams (media_info);
+  for (list = gst_player_get_video_streams (media_info);
       list != NULL; list = list->next) {
     gint fps_d, fps_n;
     guint par_d, par_n;
@@ -649,7 +644,7 @@ test_subtitle_info (GstPlayerMediaInfo * media_info)
 {
   GList *list;
 
-  for (list = gst_player_media_info_get_subtitle_streams (media_info);
+  for (list = gst_player_get_subtitle_streams (media_info);
       list != NULL; list = list->next) {
     GstPlayerStreamInfo *stream = (GstPlayerStreamInfo *) list->data;
     GstPlayerSubtitleInfo *sub = (GstPlayerSubtitleInfo *) stream;
@@ -669,7 +664,7 @@ test_media_info_object (GstPlayer * player, GstPlayerMediaInfo * media_info)
 {
   GList *list;
 
-  /* global tag */
+  /* gloabl tag */
   fail_unless (gst_player_media_info_is_seekable (media_info) == TRUE);
   fail_unless (gst_player_media_info_get_tags (media_info) != NULL);
   fail_unless_equals_string (gst_player_media_info_get_title (media_info),
@@ -685,15 +680,15 @@ test_media_info_object (GstPlayer * player, GstPlayerMediaInfo * media_info)
   fail_unless (list != NULL);
   fail_unless_equals_int (g_list_length (list), 10);
 
-  list = gst_player_media_info_get_video_streams (media_info);
+  list = gst_player_get_video_streams (media_info);
   fail_unless (list != NULL);
   fail_unless_equals_int (g_list_length (list), 1);
 
-  list = gst_player_media_info_get_audio_streams (media_info);
+  list = gst_player_get_audio_streams (media_info);
   fail_unless (list != NULL);
   fail_unless_equals_int (g_list_length (list), 2);
 
-  list = gst_player_media_info_get_subtitle_streams (media_info);
+  list = gst_player_get_subtitle_streams (media_info);
   fail_unless (list != NULL);
   fail_unless_equals_int (g_list_length (list), 7);
 
@@ -1012,7 +1007,7 @@ END_TEST;
 static gboolean
 has_subtitle_stream (TestPlayerState * new_state)
 {
-  if (gst_player_media_info_get_subtitle_streams (new_state->media_info))
+  if (gst_player_get_subtitle_streams (new_state->media_info))
     return TRUE;
 
   return FALSE;
@@ -1215,7 +1210,7 @@ START_TEST (test_play_audio_video_eos)
   gst_player_play (player);
   g_main_loop_run (state.loop);
 
-  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data) & (~0x10), 10);
+  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data) & (~0x10), 9);
 
   stop_player (player, &state);
   g_object_unref (player);
@@ -1347,21 +1342,17 @@ test_play_error_invalid_uri_and_play_cb (GstPlayer * player,
       new_state->test_data = GINT_TO_POINTER (step + 1);
       break;
     case 8:
-      fail_unless_equals_int (change, STATE_CHANGE_MEDIA_INFO_UPDATED);
-      new_state->test_data = GINT_TO_POINTER (step + 1);
-      break;
-    case 9:
       fail_unless_equals_int (change, STATE_CHANGE_DURATION_CHANGED);
       fail_unless_equals_uint64 (new_state->duration,
           G_GUINT64_CONSTANT (464399092));
       new_state->test_data = GINT_TO_POINTER (step + 1);
       break;
-    case 10:
+    case 9:
       fail_unless_equals_int (change, STATE_CHANGE_POSITION_UPDATED);
       fail_unless_equals_uint64 (new_state->position, G_GUINT64_CONSTANT (0));
       new_state->test_data = GINT_TO_POINTER (step + 1);
       break;
-    case 11:
+    case 10:
       fail_unless_equals_int (change, STATE_CHANGE_STATE_CHANGED);
       fail_unless_equals_int (old_state->state, GST_PLAYER_STATE_BUFFERING);
       fail_unless_equals_int (new_state->state, GST_PLAYER_STATE_PLAYING);
@@ -1393,7 +1384,7 @@ START_TEST (test_play_error_invalid_uri_and_play)
   gst_player_play (player);
   g_main_loop_run (state.loop);
 
-  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data), 12);
+  fail_unless_equals_int (GPOINTER_TO_INT (state.test_data), 11);
 
   stop_player (player, &state);
   g_object_unref (player);
@@ -1701,35 +1692,50 @@ player_suite (void)
   tcase_add_test (tc_general, test_set_and_get_uri);
   tcase_add_test (tc_general, test_set_and_get_position_update_interval);
 
-  /* Skip playback tests for now.
-   * See https://bugzilla.gnome.org/show_bug.cgi?id=787374 */
-
 #ifdef HAVE_VALGRIND
   if (RUNNING_ON_VALGRIND) {
   } else
 #endif
   {
-    tcase_skip_broken_test (tc_general, test_play_position_update_interval);
+    tcase_add_test (tc_general, test_play_position_update_interval);
   }
-  tcase_skip_broken_test (tc_general, test_play_audio_eos);
-  tcase_skip_broken_test (tc_general, test_play_audio_video_eos);
-  tcase_skip_broken_test (tc_general, test_play_error_invalid_uri);
-  tcase_skip_broken_test (tc_general, test_play_error_invalid_uri_and_play);
-  tcase_skip_broken_test (tc_general, test_play_media_info);
-  tcase_skip_broken_test (tc_general, test_play_stream_disable);
-  tcase_skip_broken_test (tc_general, test_play_stream_switch_audio);
-  tcase_skip_broken_test (tc_general, test_play_stream_switch_subtitle);
-  tcase_skip_broken_test (tc_general, test_play_error_invalid_external_suburi);
-  tcase_skip_broken_test (tc_general, test_play_external_suburi);
-  tcase_skip_broken_test (tc_general, test_play_forward_rate);
-  tcase_skip_broken_test (tc_general, test_play_backward_rate);
-  tcase_skip_broken_test (tc_general, test_play_audio_video_seek_done);
-  tcase_skip_broken_test (tc_general, test_restart);
-  tcase_skip_broken_test (tc_general, test_user_agent);
+  tcase_add_test (tc_general, test_play_audio_eos);
+  tcase_add_test (tc_general, test_play_audio_video_eos);
+  tcase_add_test (tc_general, test_play_error_invalid_uri);
+  tcase_add_test (tc_general, test_play_error_invalid_uri_and_play);
+  tcase_add_test (tc_general, test_play_media_info);
+  tcase_add_test (tc_general, test_play_stream_disable);
+  tcase_add_test (tc_general, test_play_stream_switch_audio);
+  tcase_add_test (tc_general, test_play_stream_switch_subtitle);
+  tcase_add_test (tc_general, test_play_error_invalid_external_suburi);
+  tcase_add_test (tc_general, test_play_external_suburi);
+  tcase_add_test (tc_general, test_play_forward_rate);
+  tcase_add_test (tc_general, test_play_backward_rate);
+  tcase_add_test (tc_general, test_play_audio_video_seek_done);
+  tcase_add_test (tc_general, test_restart);
+  tcase_add_test (tc_general, test_user_agent);
 
   suite_add_tcase (s, tc_general);
 
   return s;
 }
 
-GST_CHECK_MAIN (player)
+int
+main (int argc, char **argv)
+{
+  int number_failed;
+  Suite *s;
+  SRunner *sr;
+
+  gst_check_init (&argc, &argv);
+
+  s = player_suite ();
+  sr = srunner_create (s);
+
+  srunner_run_all (sr, CK_NORMAL);
+
+  number_failed = srunner_ntests_failed (sr);
+
+  srunner_free (sr);
+  return (number_failed == 0) ? 0 : -1;
+}
